SystemOrganization addCategory: #'Orca-Core-Cache'!
SystemOrganization addCategory: #'Orca-Core-Server'!
SystemOrganization addCategory: #'Orca-Core-Clientside'!
SystemOrganization addCategory: #'Orca-Core-Translator'!
SystemOrganization addCategory: #'Orca-Core-RemoteObjects'!

OMeta2 subclass: #OrcaOMeta2
	instanceVariableNames: 'currentMethod result inputConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

OrcaOMeta2 subclass: #OrcaAstReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

OrcaAstReader subclass: #OrcaAbstractPrinter
	instanceVariableNames: 'indentation'
	classVariableNames: 'ClassTextOutputStream JavascriptPrintStream TextOutputStream'
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'robi 1/17/2011 16:17'!
asJSString: aString
	"Use the squeak-library to encode the string.. if not found, use an easy escaping-algorithm."
	
	^ (aString respondsTo: #javascriptOn:)
		ifTrue: [
			JavascriptPrintStream ifNil: [ JavascriptPrintStream := TextStream on: '' ] ifNotNil: [ JavascriptPrintStream reset ].
			aString javascriptOn: JavascriptPrintStream.
			JavascriptPrintStream contents ]
		ifFalse: [ | tokens |
			tokens := aString findTokens: self jsEscapeChars keep: self jsEscapeChars.
			'"', (tokens inject: '' into: [ :all :each |
				all, 
				((self jsEscapeChars includes: each asCharacter) ifTrue: [ '\' ] ifFalse: [ '' ]), 
				each]), '"' ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/9/2011 20:23'!
convert: aBlock to: output
	
	aBlock value:
		output reset; yourself.
	^ output contents! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/5/2011 21:22'!
convertClass: aClass
	
	^ self
		convertClass: aClass
		retrievingFrom: [ :methodSelector |
			self convertCompiledMethod: methodSelector ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/6/2011 15:52'!
convertClass: aClass retrievingFrom: aBlock
	"Return the converted version of the whole class. aBlock takes one parameter (a CompiledMethod) and returns
	the code for the method as string. A static (reused) output-stream is used to print the code."
	
	^ self
		convert: [ :output | self convertClass: aClass to: output retrievingFrom: aBlock ]
		to: (TextOutputStream ifNil: [ TextOutputStream := OrcaTextStream for: self ])! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/12/2011 12:21'!
convertClass: aClass to: output retrievingFrom: aBlock
	
	| firstPrinted firstPrintedMemorizer |
	firstPrinted := false.
	firstPrintedMemorizer := [ :nextPartValid | nextPartValid and: [ firstPrinted or: [ firstPrinted := true. false ] ] ]. "Pretty complex. Remember, whether first line has been printed."
	output << OrcaSqueakAstToJsAst classFunction << '(' << (self asJSString: aClass name) << ',' << self space << '{' << self newline.
	self
		printPart: 'superclass' with: (self printSuperclass: aClass) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceVariables' with: (self printVariables: aClass instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classInstanceVariables' with: (self printVariables: aClass class instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classVariables' with: (self printVariables: aClass classVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classMethods' with: (self printMethodsOf: aClass class retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceMethods' with: (self printMethodsOf: aClass retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer.
	output << self newline << '})'.! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/5/2011 19:59'!
convertClasses: anArrayOfClasses
	
	^ self
		convertClasses: anArrayOfClasses
		retrievingFrom: [ :class |
			self convertClass: class ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/6/2011 15:52'!
convertClasses: anArrayOfClasses retrievingFrom: aBlock
	"Return the code for all the classes in anArrayOfClasses. aBlock takes on parameter (a class) and returns the code for the class."
	
	^ self
		convert: [ :output | output << (self
							printEach: [ :class | aBlock value: class ]
							of: anArrayOfClasses
							separatedBy: (';', self newline))]
		to: (ClassTextOutputStream ifNil: [ ClassTextOutputStream := OrcaTextStream for: self ])! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/13/2011 16:41'!
doesNotUnderstandClassName
	"This is named with a small letter to not collide with Squeak-classes in our st.-namespace.
	The convention is to name meta-slots in the namespace with small letter instead
	of underscores or something."
	
	^ 'doesNotUnderstandClass'! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/13/2011 16:41'!
doesNotUnderstandClassWithMethods: aDictionaryOfMethodNames

	| cr functions crr |
	cr := self newline, self indentation.
	crr := cr, self indentation.
	functions := TextStream with: ''.
	aDictionaryOfMethodNames keysAndValuesDo: 
		[ :aSTFunctionCallName :aJSFunctionCallName | 
			functions << aJSFunctionCallName << ':' << self space << 'function(){' << crr << 
			'return this.doesNotUnderstand_(st.Message.selector_arguments_(st.string("' << aSTFunctionCallName << '"), st.array(st.toArray(arguments)))); ' << '},' << crr].
	functions := functions contents.
	^ ((TextStream with: OrcaSqueakAstToJsAst classFunction, '("') << self doesNotUnderstandClassName 
			<< '", {' << cr <<
		'instanceMethods: {' << cr <<
			self indentation << functions << cr <<
		'}, ' << cr <<
		'classMethods: {' << cr <<
			self indentation << functions << cr <<
		'}' << self newline << 
		'})' << self newline) contents! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
indentation
	
	self subclassResponsibility! !

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
inputConverter

	^ OrcaSqueakAstToJsAst! !

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'ag 1/17/2011 10:39'!
jsEscapeChars
	
	^ #($" $\)! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
newline
	
	self subclassResponsibility! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 08:01'!
printEach: aBlock of: aCollection separatedBy: aString
	
	| result isFirst |
	isFirst := true.
	result := ''.
	aCollection do: [ :each | 
		isFirst
			ifTrue: [ isFirst := false ]
			ifFalse: [ result := result, aString ].
		result := result, (aBlock value: each)].
	^ result! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:16'!
printMethodsOf: aClass retrievingFrom: aBlock
	
	| methods |
	methods := aClass methodDict values.
	^ methods 
		ifEmpty: [ false ]
		ifNotEmpty: [
			'{', self newline,
			(self
				printEach: [ :eachmethod | | code |
					code := aBlock value: eachmethod.
					self indentation, self indentation, code key, ':', self space, code value ]
				of: methods
				separatedBy: ',', self newline),
			self newline, self indentation, '}' ].! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:43'!
printPart: aString with: aValue to: output guardedBy: aGuardBlock
	"Print a slot. If aValue is a String (should return false then), this is skipped.
	aGuardBlock is called with whether this part will be printed and returns, whether we have to prepend a colon + newline ('guards' whether we are the first line)."
	
	(aGuardBlock value: aValue isString)
		ifTrue: [ output << ',' << self newline ].
	aValue isString ifTrue: [
		output << self indentation << aString << ':' << self space << aValue ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/13/2011 16:41'!
printSuperclass: aClass
	
	^ OrcaSqueakAstToJsAst prefixForGlobals, 
		(aClass superclass
			ifNil: [ self doesNotUnderstandClassName ]
			ifNotNil: [ aClass superclass name ])! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/12/2011 12:29'!
printVariables: aCollection
	"Print a collection containing all members of aCollection separated by colons"
	
	"TODO its not good, that the printer accesses the escape-char from the ast-to-ast"
	
	^ aCollection
		ifEmpty: [ false ]
		ifNotEmpty: [
			'[', self space,
			(self
				printEach: [ :each | '"', OrcaSqueakAstToJsAst escapeCharForIdentifier, each, '"' ]
				of: aCollection
				separatedBy: ',', self space),
			self space, ']' ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/10/2011 09:10'!
resetTextStreams
	"These helper-TextStreams are recreated lazily"
	
	ClassTextOutputStream := TextOutputStream := JavascriptPrintStream := nil.! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
space
	
	self subclassResponsibility! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
arguments =
	
	translate*:identifiers
	-> [ self functionParameters: identifiers ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
array = 
	
	translateWith(#expressionBase)*:contents
	-> [ '[', (contents separatedBy: ', '), ']' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
assignment =
	
	translate! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
cSpace
	
	^ self class space! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
comment =
	
	^string:comment
	-> [ self createComment: comment ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
comments =
	
	translateNamed('comment')*:commentStrings
	-> [commentStrings
		ifNil: [''] 
		ifNotNil: 
			[commentStrings separatedBy: '']]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
createComment: aString

	| result |
	self indent.
	result := TextStream with: ''.
	result << self indentation << '// '.
	aString do: 
			[ :each |
			each = Character cr
				ifTrue: [result << self newline << self indentation <<  '//']
				ifFalse: [
					each = Character lf
						ifFalse: [result << each]]].
	result << self newline.
	self unindent.
	^ result contents! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
expression =
	
	expressionBase:expr
	-> [ self indentation, expr ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression: anExpressionBodyString withAssignments: aStringCollection
	
	^ ((aStringCollection isNil or: [ aStringCollection isEmpty ])
			ifTrue: [ '' ]
			ifFalse: [
				(aStringCollection separatedBy: self cSpace, '=', self cSpace),
				self cSpace, '=', self cSpace ])
					, anExpressionBodyString! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression: anExpressionBodyString withAssignments: aStringCollection withComments: aStringOfComments
	
	^ (aStringOfComments ifNil: ['']),
		((aStringCollection isNil or: [ aStringCollection isEmpty ])
			ifTrue: [ '' ]
			ifFalse: [
				(aStringCollection separatedBy: self cSpace, '=', self cSpace),
				self cSpace, '=', self cSpace ])
					, anExpressionBodyString! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
expressionBase =
	
	translateNamed('comments')?:comments translate:expressionBody translate*:assignments
	-> [ self expression: expressionBody withAssignments: assignments withComments: comments ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionCall =
	// Can be either an invokation of a named function of an object OR an invokation of a anonymous function
	
	&anything:receiverNode (nil | translate):receiver (^string | translate):functionName translate:arguments
	-> [ self
			functionCall: functionName
			to: ((receiverNode notNil and: [ receiverNode name = 'functionDeclaration' ])
					ifTrue: [ '(', receiver, ')' ]
					ifFalse: [ receiver ] )
			withArguments: arguments ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionCall: aFunctionNameOrDeclarationString to: aReceiverString withArguments: anArgumentsString
	
	| function |
	function := (aFunctionNameOrDeclarationString startsWith: 'function') "no test yet, necessary?"
		ifTrue: ['(', aFunctionNameOrDeclarationString, ')'] 
		ifFalse: [ aFunctionNameOrDeclarationString].
	
	^ (aReceiverString
		ifNotNil: [ aReceiverString, '.' ]
		ifNil: [ '' ]), function, anArgumentsString! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionContent =
	// This is intentionally not named 'statements', to deny all nested statements-nodes
	// The ast-to-ast step must flatten all statements into one list of expressions here
	
	[ self indent ]
	translate*:expressions
	[ self unindent ]
	-> [ expressions separatedBy: ';', self newline ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration =
	
	translate:arguments
	translate:statements
	-> [ self functionDeclaration: arguments withBody: statements ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration: aParameterString withBody: aStatementsString
	"aParameterString already contains the parantheses; #functionParameters: should be used."
	
	^ 'function', self cSpace, aParameterString, self cSpace, '{', self newline,
			(aStatementsString ifEmpty: [ '' ] ifNotEmpty: [ aStatementsString, ';' ]),
		self newline, self indentation, '}'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration: aParameterString withComment: aCommentString withBody: aStatementsString

	"aParameterString already contains the parantheses; #functionParameters: should be used."
	^ 'function', self cSpace, aParameterString, self cSpace, '{', self newline,
			(aCommentString ifNil: [ '' ]),
			(aStatementsString ifEmpty: [ '' ] ifNotEmpty: [ aStatementsString, ';' ]),
		self newline, self indentation, '}'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionParameters: aStringCollection

	^ '(', ((aStringCollection collect: [ :aString | aString trimLeft ]) separatedBy: ', '), ')'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
hasSelector: aSymbol
	"Adding the superclass method dictionary to the searcharea for matching methods is required because the printers don't implement all the methods on their own"
	
	^ ((super hasSelector: aSymbol) or: [ self class superclass includesSelector: aSymbol ])
		ifTrue: [ true ]
		ifFalse: [ self error: 'Unknown selector: ', aSymbol asString. false ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
header =
	
	^string:functionName arguments:arguments
	-> [ functionName -> arguments ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
identifier =
	
	^string:identifier
	-> [ identifier ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
indent
	
	self indentation: self indentation, self class indentation! !

!OrcaAbstractPrinter methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
indentation
	^ indentation! !

!OrcaAbstractPrinter methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
indentation: anObject
	indentation := anObject! !

!OrcaAbstractPrinter methodsFor: 'initialization' stamp: 'lw 4/6/2011 17:59'!
initialize

	super initialize.
	self 
		indentation: ''.! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
javaScriptMethod =
	// This does not return a string, but an association from the method-name to the method-declaration
	
	translate:functionNameAndArguments
	translate:statements
	-> [ functionNameAndArguments key -> 
		(self
			functionDeclaration: functionNameAndArguments value
			withBody: statements) ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
method =
	// This does not return a string, but an association from the method-name to the method-declaration
	
	translate:functionNameAndArguments
	translateNamed('comments')?:comment
	translate:statements
	-> [ functionNameAndArguments key -> 
		(self
			functionDeclaration: functionNameAndArguments value
			withComment: comment
			withBody: statements) ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
newExpression =
	
	// Here is a function-call, which is handled a little bit differently. Code copied from functionCall.
	
	{ &anything:receiverNode (nil | translate):receiver (^string | translate):functionName translate:arguments }
	-> [ 'new (', (self
			functionCall: functionName, ')'
			to: ((receiverNode notNil and: [ receiverNode name = 'functionDeclaration' ])
					ifTrue: [ '(', receiver, ')' ]
					ifFalse: [ receiver ] )
			withArguments: arguments) ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
newline
	
	^ self class newline! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
number =
	
	^number:number
	-> [ number asString ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
objectLiteral =

	translateNamed(#slotAssignment)*:result
	-> [ '{', (result separatedBy: ',', self cSpace) , '}' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
paranthesizedExpression =
	
	expressionBase:value
	-> [ '(', value, ')' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
reservedWord =
	
	^string! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
returnExpression =
	
	expressionBase:expr
	-> [ self indentation, 'return ', expr ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
slotAccess =
	
	(nil | translate):object ^symbol:slotName
	-> [ (object ifNotNil: [ object, '.' ] ifNil: [ '' ]), slotName asString ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
slotAssignment =
	
	^symbol:slotname translate:value
	-> [  slotname, self cSpace, ':', self cSpace, value ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
string =
	
	^string:string
	-> [ self class asJSString: string ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
unindent
	
	self indentation: (self indentation allButLast: self class indentation size)! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
variableDeclaration =
	
	translateNamed('identifier')+:variableNames translateNamed('assignment')?:assignment
	-> [ self variableDeclaration: variableNames withAssignment: assignment ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
variableDeclaration: variableNames withAssignment: assignment
	"Acts as an expression, not terminated by a semi colon"
	
	| oneVar firstLine |
	firstLine := self indentation, 'var ', ((oneVar := variableNames size = 1)
		ifTrue: [ variableNames first ]
		ifFalse: [ (variableNames separatedBy: ';', self cSpace, 'var ')]).
	^ assignment 
		ifNil: [ firstLine ]
		ifNotNil: [
			oneVar
				ifTrue: [ self expression: assignment withAssignments: { firstLine } ]
				ifFalse: [ 
					firstLine, ';', self newline, self indentation, (self expression: assignment withAssignments: variableNames) ]]! !

OrcaAbstractPrinter subclass: #OrcaPrettyPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ '    '! !

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'sec 3/2/2011 16:08'!
newline
	
	^ Character lf asString! !

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
space

	^ ' '! !

!OrcaPrettyPrinter methodsFor: 'initialization' stamp: 'lw 4/6/2011 17:59'!
initialize
	"Do everything indented x2 from the start, to have json-like pretty-print for the whole js-file."
	
	super initialize.
	self
		indent;
		indent.! !

OrcaAbstractPrinter subclass: #OrcaPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ ''! !

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:02'!
newline
	
	^ ''! !

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:30'!
space

	^ ''! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:20'!
applyRule: aRule on: aMatcher
	
	^ super applyRule: #translate on: aMatcher! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 4/6/2011 18:12'!
convertAst: anOrcaNodeTree

	| matcher result |
	matcher := self matcherOn: {anOrcaNodeTree} readStream.
	matcher currentMethod: nil.
	result := self applyRule: #translate on: matcher.
	^ result! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:21'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ self inputConverter convert: aString as: aRule insideMethod: aCompiledMethod.! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 12/8/2010 22:13'!
inputConverter
	"An AstReader should know, which converter-class can provide the ast wants to read"
	
	self subclassResponsibility! !

!OrcaAstReader methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
hasSelector: aSymbol
	"Offers a hook to catch, if an unknown node occurs"
	
	^ self class includesSelector: aSymbol! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translate =
	// Translate the next node using it's name as rule
	
	translateNamed(nil)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamed :nodeName =
	// Assert, that the next node has the name nodeName and translate it using it's name as rule
	
	translateNamedWith(nodeName. nodeName)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamedWith :nodeName :targetRule =
	// Assert, that the next node has the name nodeName and translate it with targetRule
	
	translateNamedWithArgument(nodeName. targetRule. nil)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamedWithArgument :nodeName :targetRule :argument =
	// Assert, that the next node has the name nodeName, use targetRule to convert the node and pass argument when calling targetRule
	
	// Lookahead for one node
	&anything:node
	
	// Some assertions, e.g. about the nodeName
	?[ node class = OrcaNode ]
	?[ nodeName isNil | (node name = nodeName )]
	[ targetRule ifNil: [ node name asSymbol ] ifNotNil: [ targetRule asSymbol ]]:appliedRule
	?[ self hasSelector: appliedRule ]
	
	// If an argument is given, call the targetRule with that argument. Else, call the targetRule without arguments
	(( ?[ argument notNil ] { apply(appliedRule. argument):result } )
	| { apply(appliedRule):result })
	
	// Return the parsed content of the node
	-> [ result ]! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateWith :aRule =
	// Translate the next node with aRule, regardless of it's name
	
	translateNamedWith(nil. aRule)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateWithArgument :aRule :argument =
	// Translate the next node with aRule, regardless of it's name, and pass aRule the argument
	
	translateNamedWithArgument(nil. aRule. argument)! !

OrcaAstReader subclass: #OrcaAstToAst
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
passThroughAndTranslate =
	
	// TODO add all primitive values?..
	
	^symbol
|	string
|	number
|	char
|	nil
	
|	({ translate*:children }:node
	-> [ OrcaNode named: node name with: children ])! !

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
renameTo :nodeName =
	
	translate*:contents
	-> [ OrcaNode named: nodeName with: contents ]! !

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
translate = 
	
	^translate
|	passThroughAndTranslate! !

OrcaAstToAst subclass: #OrcaJsAstOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaJsAstOptimizer class methodsFor: 'converting' stamp: 'ag 4/6/2011 18:12'!
inputConverter
	
	self error: 'The Js-Ast-Optimizer is not yet functional!!'.
	^ OrcaSqueakAstToJsAst! !

!OrcaJsAstOptimizer methodsFor: 'convert special cases' stamp: 'lw 4/6/2011 17:59'!
forLoop =
	// Special case of keywordExpression
	

	translate:receiver symbol:selector translate*:arguments
	?[ selector = #do: ]
	?[ arguments size = 1 ]
	[ arguments first ]:block
	?[ block size >= 1 ]
	?[ block first name = 'arguments' ]
	?[ block first size = 1 ]
	-> [ self forLoopOver: receiver do: block ]! !

!OrcaJsAstOptimizer methodsFor: 'convert special cases' stamp: 'lw 4/6/2011 17:59'!
forLoopOver: anArrayNode do: aBlockNode

	| arrayName statements aStatementsNode aTemporariesNode loopVariableName |
	arrayName := 'arr$'.
	loopVariableName := aBlockNode first first.
	aStatementsNode := aBlockNode detect: [:each | each name = 'statements'] ifNone: [ nil ].
	aTemporariesNode := aBlockNode detect: [:each | each name = 'temporaries'] ifNone: [ nil ].
	aStatementsNode ifNotNil: [ statements := self apply: #translate withArgs: aStatementsNode].
	^ OrcaNode statements:
		{OrcaNode variableDeclaration:
			{OrcaNode identifier: arrayName.
			  OrcaNode expression: anArrayNode}.
		 OrcaNode forLoop:
			{OrcaNode identifier: arrayName.
			loopVariableName.
			aTemporariesNode.
			statements}}! !

!OrcaJsAstOptimizer methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpression = 
	
	forLoop
|	messageSendExpression // ...not correct ...
! !

OrcaAstToAst subclass: #OrcaSqueakAstToJsAst
	instanceVariableNames: 'calledFunctions referedClasses temps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
arrayFunction
	
	^ self prefixForGlobals, 'array'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
blockFunction
	
	^ self prefixForGlobals, 'block'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:10'!
charFunction
	
	^ self prefixForGlobals, 'char'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 21:11'!
classFunction
	
	^ self prefixForGlobals, 'class'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForIdentifier
	
	^ '$'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForMethod
	
	^ '_'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:21'!
escapeGlobal: aStringOrSymbol
	"This only gets called, if it's clear, that aStringOrSymbol is really a class"
	
	^  self prefixForGlobals, aStringOrSymbol asString! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:19'!
escapeReservedWord: aStringOrSymbol
	
	aStringOrSymbol ifNil: [ ^nil ].
	^ ((self jsReservedWords includes: aStringOrSymbol asString)
		ifTrue: [ self escapeCharForMethod ]
		ifFalse: [ '' ]),
			aStringOrSymbol asString! !

!OrcaSqueakAstToJsAst class methodsFor: 'converting' stamp: 'LT 3/1/2011 10:10:30.562'!
ignoredPrimitives
	"This returns an array of CompiledMethod. All primitive-declarations in these methods are ignored.
	This means, these methods must have a correct software-fallback-implementation."
	
	^ {
	self method: #@ of: Number.
	self method: #replaceFrom:to:with:startingAt: of: String.
	self method: #replaceFrom:to:with:startingAt: of: Symbol.
	self method: #replaceFrom:to:with:startingAt: of: Array.
	self method: #replaceFrom:to:with:startingAt: of: ByteString.
	self method: #atEnd of: PositionableStream.
	self method: #next of: ReadStream.
	self method: #species of: Object
	}! !

!OrcaSqueakAstToJsAst class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
inputConverter
	
	^ OrcaParser! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 13:46'!
jsFunctionNameFor: aStringOrSymbol
	
	| result |
	result := ''.
	aStringOrSymbol asString do: [ :each | | jsIdentifier |
		jsIdentifier := self squeakCharacterToJsIdentifier at: each ifAbsent: [each asString].
		result := result, jsIdentifier].
	^ self escapeReservedWord: result! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 13:28'!
jsIdentifierFor: aString inMethod: aCompiledMethod withTemps: temps
	"Escape reserved js-words and instance variables."
	
	^ (temps includes: aString)
	ifTrue: [ 
		"Temporary or method-argument"
		OrcaNode named: 'identifier' with: { self escapeReservedWord: aString } ]
	ifFalse: [
		(aCompiledMethod notNil and: [ aCompiledMethod methodClass allInstVarNames includes: aString ])
			ifTrue: [
				"Access to instance/class-variable"
				OrcaNode named: 'slotAccess' with: { 
					OrcaNode named: 'reservedWord' with: { 'this' }. 
					(self jsInstVarNameFor: aString) asSymbol } ]
			ifFalse: [
				"Access to global"
				OrcaNode named: 'identifier' with: { self escapeGlobal: aString } ] ]! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 3/29/2011 10:46'!
jsInstVarNameFor: aString
	
	^ self escapeCharForIdentifier asString, aString! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 12:35'!
jsKeywordFor: aString
	"thisContext is not covered, just a stub leading to a not-defined-error.
	The returned association maps from the literal evaluating to the squeak-object to the literal evaluating to the underlying js-object."
	
	^ aString
		caseOf: {
			[ 'self' ] -> [ 'this' ].
			[ 'true' ] -> [ self prefixForGlobals, 'true' ].
			[ 'false' ] -> [ self prefixForGlobals, 'false' ].
			[ 'nil' ] -> [ self prefixForGlobals, 'nil' ].
			"In Squeak, using super as identifier (e.g. a := super. ) without calling functions on it, seems to have the same effect as using self."
			"Calling super-functions is not handled here"
			[ 'super' ] -> [ 'this' ].
			"thisContext is not implemented in any way"
			[ 'thisContext' ] -> [ 'THIS_CONTEXT' ].
		}
		otherwise: [ self error: 'Cannot translate keyword to JS: ', aString ]! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:30'!
jsReservedWords
	"http://javascript.about.com/library/blreserved.htm"
	
	^ #('abstract' 'as' 'boolean' 'break' 'byte' 'case' 'catch' 'char' 'class' 'continue' 'const' 'debugger' 'default' 'delete' 'do' 'double' 'else' 'enum' 'export' 'extends' 'final' 'finally' 'float' 'for' 'function' 'goto' 'if' 'implements' 'import' 'in' 'instanceof' 'int' 'interface' 'is' 'long' 'namespace' 'native' 'new' 'null' 'package' 'private' 'protected' 'public' 'return' 'short' 'static' 'super' 'switch' 'synchronized' 'this' 'throw' 'throws' 'transient' 'try' 'typeof' 'use' 'var' 'void' 'volatile' 'while' 'with' 'arguments'
	) 
	asOrderedCollection
		add: self namespaceForGlobals;
		yourself! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:31'!
namespaceForGlobals

	^ 'st'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:09'!
nonLocalReturnFunction
	
	^ self prefixForGlobals, 'nonLocalReturn'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
numberFunction
	
	^ self prefixForGlobals, 'number'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 16:28'!
prefixForCommunication
	
	^ self prefixForGlobals, 'communication.'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 12:32'!
prefixForGlobals

	^ self namespaceForGlobals, '.'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
primitiveCallFunction
	
	^ self prefixForGlobals, 'primitiveDeclaration'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/12/2011 16:26'!
serverBlockFunction
	
	^ self prefixForCommunication, 'serverBlock'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'lw 2/17/2011 16:33'!
squeakCharacterToJsIdentifier
	"Several characters can be used for selectors in squeak but
	are not allowed for JavaScript functionname"
	
	"TODO check, if one of these might be a reserved word (would cause conflicts with unary message named that way). Builds exception-rule."
	
	^ (Dictionary newFrom:
		{$: -> ''.  
		$+ -> 'plus'.
		$- -> 'minus'.
		$* -> 'times'.
		$/ ->  'slash'.
		$\ -> 'backslash'.
		$~ -> 'tilde'.
		$< ->  'less'.
		$> -> 'greater'.
		$= -> 'equals'.
		$@ -> 'at'.
		$% -> 'percent'.
		$& -> 'and'.
		$? -> 'question'.
		$!! -> 'exclamation'.
		$` -> 'backtick'.
		$, -> 'comma'.
		$ -> 'plusminus'.
		$| -> 'pipe'})
			collect: [ :string | self escapeCharForMethod, string ]! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:01'!
stringFunction
	
	^ self prefixForGlobals, 'string'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:05'!
superFunction
	
	^ self prefixForGlobals, 'super'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:23'!
unboxSlotObjectFunction
	
	^ self prefixForGlobals, 'unboxSlotObject'! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/11/2011 12:36'!
addCollectedClass: aClass

	self referedClasses add: aClass	
	! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
addReturnExpression: statements
	
	(statements isEmpty or: [(statements last name = 'returnExpression') not])
		ifTrue: [
			statements add: (OrcaNode returnExpression: { OrcaNode reservedWord: 'this' })].! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:41'!
arguments =
	
	lookaheadTemps
	renameTo('arguments')! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
array =
	// Pack inner nodes in expression-nodes
	
	translate*:elements
	arrayBase(elements collect: [ :each | OrcaNode expression: { each }])! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
arrayBase :collection =
	
	wrapAsPrimitive(
		self class arrayFunction.
		'array'.
		collection)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpression =
	
	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
block =
	
	translateNamed('arguments')?:arguments
	methodContent:statements
	
	// All return-statements in the block are nonlocal returns. After that, the last expression into a return-expression like in a method.
	// Nonlocal returns are function calls in JS: nonLocalReturn()
	[ self renameNonLocalReturns: statements ]
	renameLastExpression(statements)
	
	[OrcaNode named: 'functionDeclaration' withoutNils: 
		{( arguments 
			ifNotNil: [ arguments ] 
			ifNil: [ OrcaNode arguments ] ).
		statements }]:blockContent
	
	-> [(OrcaNode functionCall: 
		{ nil.
		self class blockFunction.
		OrcaNode arguments: {blockContent}}) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 12:04'!
braceExpression =
	// take the contents of the expression as it's js-arg-value. In a brace-expression, there is no assigment.
	
	translate*:expressions
	arrayBase(expressions)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
byteArray =
	
	array! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/11/2011 12:37'!
calledFunctionNamesAt: aStringOrSymbol put: anotherStringOrSymbol 

	(aStringOrSymbol isString
			or: [aStringOrSymbol isSymbol])
		ifTrue: [^ self calledFunctions 
					at: aStringOrSymbol asSymbol 
					put: anotherStringOrSymbol asSymbol]! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
calledFunctions
	^ calledFunctions! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
calledFunctions: anObject
	calledFunctions := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
cascadeReceiver
	
	^ OrcaNode identifier: ('cascadeReceiver', self class escapeCharForIdentifier)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression =
	
	translate:receiver
	translateWithArgument(#keywordExpression. self cascadeReceiver)*:functions
	-> [ self cascadedExpression: functions to: receiver ]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/12/2011 14:15'!
cascadedExpression: functions to: receiver
	"The statements-node created here is later flattened into one flat list of expressions."
	
	| statements |
	statements := OrcaNode functionDeclaration:
		{ OrcaNode arguments. 
		OrcaNode functionContent: 
			{ OrcaNode variableDeclaration: 
				{ self cascadeReceiver. 
				OrcaNode assignment: { receiver }}}}.
	statements last addAll: (functions collect: [ :each | self expression: OrcaNode expression add: each ]).
	statements last last name: 'returnExpression'.
	^ self
		functionCall: 'apply'
		to: statements
		withArguments: { OrcaNode reservedWord: 'this' }! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
character =
	
	char:char
	wrapAsPrimitive(self class charFunction. 'string'. char asString) // the 'string' here is the node inside the character()-function call. The Printer does not know about characters.
	! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
checkSingleArgumentSelector: aSymbol
	"Answer, whether aSymbol is a selector for a keyword-message with one argument.
	aSymbol is already is keyword-selector.
	TODO Please implement a better way to do this."
	
	| tokens |
	tokens := aSymbol findTokens: $:.
	^ tokens size = 1 and: [ tokens first = (aSymbol asString subStrings: ':') first ]! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/11/2011 12:37'!
collectReferedClasses: aString

	(Smalltalk classNamed: aString asSymbol) 
		ifNotNil: [ :aClass | self addCollectedClass: aClass ]
	
	! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
comment =
	//Neccessary to avoid packing String into calls to the string-wrapper function
	^string:string
	
	-> [ OrcaNode comment: string asString ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
comments =
	
	anything:comments
	?[ comments name = 'comments' ]
	(?[comments isEmpty] -> [nil]
|	-> [ comments ])
	! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
doItSnippet =
	// No need to turn the last expression into a returnExpression, like in a method
	
	methodContent! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression =

	comments?:comments
	translate*:nodes
	->[ (comments
			ifNil: [ nodes ]
			ifNotNil: [ {comments}, nodes  ])
		inject: OrcaNode expression 
		into: [ :newCollection :each | self expression: newCollection add: each ]]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/12/2011 14:15'!
expression: newCollection add: each
	"kein double dispatch, da Nodes namen haben und nicht nach klassen aufgeteilt sind."
	
	each name = 'returnExpression'
		ifTrue: 
			[ ^each ].
	each name = 'expression'
		ifTrue: 
			[ ^ newCollection 
				addAllLast: each;
				yourself ].
	^ newCollection 
		addLast: each; 
		yourself! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/11/2011 20:05'!
functionCall: selector to: receiver withArguments: arguments
	"In case of a super-call: the functionName is a _super-function-call itself: _super('field')() "
	
	| functionName |
	functionName := selector isString ifTrue: [ self class jsFunctionNameFor: selector ] ifFalse: [ selector ].
	^ OrcaNode functionCall: {
		receiver.
		functionName.
		OrcaNode arguments: arguments }.
	
	! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 4/12/2011 17:57'!
identifier =
	
	^string:name
	[ self collectReferedClasses: name ]
	-> [ self class
		jsIdentifierFor: name
		inMethod: self currentMethod 
		withTemps: self temps ]! !

!OrcaSqueakAstToJsAst methodsFor: 'initialization' stamp: 'ag 4/12/2011 17:57'!
initialize

	super initialize.
	self calledFunctions: Dictionary new.
	self referedClasses: Set new.
	self temps: Set new.
	self temps add: self cascadeReceiver first. "The cascadeReceiver$-variable must not be prepended with the namespace or escaped!!"! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/11/2011 15:46'!
keywordExpression =

	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/12/2011 13:39'!
lookaheadTemps =
	// This does not change the input stream.
	
	&(anything*):identifiers
	[ self temps addAll: (identifiers collect: [ :each | each first ]) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:40'!
messagePattern =
	
	^symbol:selector lookaheadTemps translate*:arguments
	-> [ OrcaNode named: 'header' with: { self class jsFunctionNameFor: selector }, arguments ]! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/11/2011 20:06'!
messageSendExpression =
	
	superCall
	
|	(translate:receiver ^symbol:selector translate*:arguments
	
	// TODO refactor; copied from #functionCall:to:withArguments:
	[ 
	self calledFunctionNamesAt: selector put: 
			(selector isString ifTrue: [ self class jsFunctionNameFor: selector ] ifFalse: [ selector ]) ]
	
	-> [ self functionCall: selector to: receiver withArguments: arguments ])! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
method =
	
	translateWith(#messagePattern):header
	comments?:comments
	methodContent:statements
	[ self addReturnExpression: statements ]
	
	-> [ OrcaNode named: 'method' withoutNils: { header. comments. statements }]! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
methodContent  =
	// This can also be used in a place, where primitive-declarations are not allowed: the parser already handles that correctly
	// This flattens all expressions (cascaded expressions in the first place) into one flat collection of expressions

	translateNamed('temporaries')?:temporaries
	translateNamed('primitiveDeclaration')*:primitives
	translate?:statements
	-> [ self statements: statements withTemps: temporaries andPrimitives: primitives ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
number =
	// Number-literals are packed into a 'primitive' number-function-call
	
	^number:number
	wrapAsPrimitive(self class numberFunction. 'number'. number)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 12:03'!
paranthesizedExpression =
	// I just found this method empty (without comment), don't really remember, why this works, but it does.
	
	translate! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclaration =
	// Some primitive-declarations are just ignored, because the software implementation is perfectly fine.
	
	(?[ self class ignoredPrimitives includes: self currentMethod ]
	anything*
	-> [ nil ])
|	
	(anything // ignored for now. No need to remember the primitive-number, the js-implementation will know which method this is in
	-> [ OrcaNode expression:
		{ self functionCall: self class primitiveCallFunction to: nil withArguments: {} } ])! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/11/2011 12:34'!
referedClasses
	^ referedClasses! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/11/2011 12:34'!
referedClasses: anObject
	referedClasses := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/11/2011 21:43'!
renameLastExpression :statements =
	
	[ statements ifNotEmpty: [ 
		(statements last name = 'expression'
			and: [statements last first name = 'functionCall']
				and: [statements last first second = 'nonLocalReturn'])   
		 			ifFalse: [ statements last name: 'returnExpression' ] ] ]
																				// WOW this is a NICE one!!!! LAST FIRST SECOND !!!!!!
! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/11/2011 16:12'!
renameNonLocalReturns: statements
	
	^ statements doWithIndex: [ :st :index |
		st name = 'returnExpression' 
			ifTrue: [ | contents |
				st name: 'expression'.
				contents := st first.
				st removeAll; add: (
					self
						functionCall: self class nonLocalReturnFunction
						to: nil
						withArguments: { contents }) ]]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 4/12/2011 12:17'!
reservedWord =
	
	^string:keyword
	-> [ OrcaNode reservedWord: (self class jsKeywordFor: keyword) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 4/11/2011 21:12'!
serverBlock =
	
	&anything:block
	translateNamed('block') ^symbol:selector
	?[ selector = #onServer ]
	-> [ self
		functionCall: self class serverBlockFunction
		to: nil
		withArguments: { OrcaNode string: block originalCode } ]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
statements: aNode withTemps: anElement andPrimitives: anElementCollection
	"All nils in the primitives-collection are removed (can occur, see #primitiveDeclaration)"
	
	| result |
	result := aNode ifNil: [ OrcaNode statements ] ifNotNil: [ aNode ].
	anElement ifNotNil: [ result addFirst: anElement; yourself ].
	anElementCollection ifNotNil: [
		result addAllFirst: (anElementCollection reject: [ :each | each isNil ]); yourself ].

	^ result name: 'functionContent'; yourself! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
string =
	// String/Symbol-literals are packed into a 'primitive' string-function-call
	
	^string:string // Also covers symbols
	wrapAsPrimitive(self class stringFunction. 'string'. string asString)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'lw 4/6/2011 17:59'!
superCall =
	// An exception to a message-send-expression: super-calls are translated in this way.
	
	{ ^string:superKeyword }:superNode ^symbol:selector translate*:arguments
	?[ superNode name = 'reservedWord' & superKeyword = 'super' ]
	-> [ 
		self
			functionCall: 
				(self
					functionCall: self class superFunction
					to: nil
					withArguments: { OrcaNode string: (self class jsFunctionNameFor: selector asString) })
			to: nil
			withArguments: arguments ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
symbol =
	
	string! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
symbolNode =
	
	^anything:symbolNode
	?[ symbolNode name  = 'symbol']
	-> [symbolNode at: 1]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:38'!
temporaries =
	
	lookaheadTemps
	
	renameTo('variableDeclaration'):temporaries
	[ temporaries ifEmpty: [ nil ] ifNotEmpty: [ temporaries ] ]! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'ag 4/11/2011 20:47'!
temps
	^ temps! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'ag 4/11/2011 20:47'!
temps: anObject
	temps := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpression =

	serverBlock // [ 'a + b' ] onServer value   -->   onServer("a + b").value()
|	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/12/2011 12:05'!
wrapAsPrimitive :functionName :nodeName :wrappedObject =
	// Wrap an object into a node, that will result in a function-call to functionName with the object as argument
	
	[ OrcaNode
		named: nodeName
		with: ( wrappedObject isCollection & wrappedObject isString not
			ifTrue: [ wrappedObject ]
			ifFalse: [ { wrappedObject } ]) ]:primitiveObject
	-> [ self
		functionCall: functionName
		to: nil
		withArguments: { primitiveObject } ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:34'!
allMethodsDo: aBlock
	"Execute aBlock with each parsable method in the image and return the sum of visited methods and classes."
	
	| numberOfObjects |
	numberOfObjects := 0.
	Smalltalk allClasses do: [ :eachClass |
		eachClass methodDict notEmpty  ifTrue: [
			numberOfObjects := numberOfObjects + 1.
			eachClass methodDict do: [ :eachMethod |
				numberOfObjects := numberOfObjects + 1.
				aBlock value: eachMethod ]]].
	^ numberOfObjects! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/15/2010 16:25'!
allMethodsDoAndReport: aBlock
	"Execute aBlock for each method in the image, print which methods have failed
	and notify each 500 methods. Print status-report at the end."
	
	| converted succeeded total |
	converted := -1.
	succeeded := 0.
	total := Smalltalk allClasses inject: 0 into: [ :sum :each | sum + each methodDict size].
	Transcript cr; show: 'Total methods in image: ', total asString.
	[
		self allMethodsDo: 
			[ :eachMethod |
				[ converted := converted + 1. aBlock value: eachMethod. succeeded := succeeded + 1.
				(converted \\ 500 = 0) ifTrue: [ Transcript cr; show: 'Converted ', converted asString, ' methods.' ]
				]
					on: Exception do: [ :err | Transcript cr; show: '	', converted asString, ': ', eachMethod name, ' (', err, ')' ]].
	]
		ensure: [ Transcript cr; show: 'Finished!! ', succeeded asString, ' of ', converted asString, ' operations succeeded!!' ].! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:52'!
applyRule: aRule on: aMatcher

	^ aMatcher apply: aRule! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/13/2010 14:34'!
classNamed: aClassOrClassname
	
	^ Smalltalk classNamed: aClassOrClassname asString! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:14'!
convert: aString as: aRule
	
	^ (self convert: aString as: aRule insideMethod: nil) result! !

!OrcaOMeta2 class methodsFor: 'converting' stamp: 'ag 4/13/2011 14:18'!
convert: aString as: aRule insideMethod: aCompiledMethod
	"Translate aString in a new parse using aRule. aCompiledMethod is additional contextual information, that influences
	e.g. printing of literals, if the class of aCompiledMethod contains instance-variables. It can be nil.
	Return the finished parser, that contains information about the parsing process."
	
	| matcher result input |
	matcher := self new 
		inputConverter: (self createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod); 
		yourself.
	input := matcher inputConverter isString 
				ifTrue: [ matcher inputConverter ] 
				ifFalse: [ { matcher inputConverter result } ]. "The outermost Collection (former an S2JNode) is not evaluated by Ometa "
	matcher 
		initInput: (OM2LazyStream for: input readStream withPos: 1);
		currentMethod: aCompiledMethod.
	result := self applyRule: aRule on: matcher.
	^ matcher finalize; 
			result: result; 
			yourself! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'ag 12/15/2010 16:26'!
convertAllMethods
	"Convert all methods in this image."
	
	self allMethodsDoAndReport: [ :eachMethod | 
		self convertCompiledMethod: eachMethod].! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'LT 3/16/2011 14:07:52.228'!
convertAllMethodsOf: aClass
	
	^ aClass methodDict notEmpty
		ifTrue: [ | methods |
			methods := Dictionary new.
			aClass methodDict valuesDo:
				[ :method | | result |
					[ result := self convertCompiledMethod: method ]
						on: Exception do: [ :err | self error: 'Failed to convert ', method printString, ': ', err ].
					methods at: result key put: result value].
			methods]
		ifFalse: [ self error: 'Cannot convert class ', aClass name ]! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:14'!
convertCompiledMethod: aCompiledMethod
	
	^ (self converterOfCompiledMethod: aCompiledMethod) result! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:15'!
convertDoIt: aString
	
	^ self convert: aString as: #doItSnippet! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:15'!
convertMethod: aString
	
	^ self convert: aString as: #method! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:16'!
convertMethod: aStringOrSymbol of: aClassOrClassname
	
	^ self convertCompiledMethod: (
		self
			method: aStringOrSymbol
			of: aClassOrClassname)! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'ag 4/6/2011 15:52'!
convertRandomMethods: aNumber
	"Match some random methods from the image. Return a String summarizing the result.
	Failed methods are printed to the Transcript."
	
	| runs successes |
	runs := successes := 0.
	aNumber timesRepeat: 
		[ | method |
			method := self randomMethod.
			method ifNotNil: [
				[ runs := runs + 1. OrcaParser convertCompiledMethod: method. successes := successes + 1 ]
					on: Error do: [ :err | Transcript cr; cr; show: 'Failed to parse: ', method name, '. ', err ]]].
	^ runs asString, ' runs, ', successes asString, ' successes'! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:18'!
convertStatements: aString
	
	^ self convert: aString as: #statements! !

!OrcaOMeta2 class methodsFor: 'converting' stamp: 'ag 4/13/2011 14:19'!
converterOfCompiledMethod: aCompiledMethod
	"Convert aCompiledMethod and return the parser, that contains the parsing results."
	
	^ self
		convert: (
			(aCompiledMethod methodClass compilerClass == Compiler)
				ifTrue: [ self sourceOf: aCompiledMethod ]
				ifFalse: [ self decompiledSourceOf: aCompiledMethod ])
		as: #method
		insideMethod: aCompiledMethod! !

!OrcaOMeta2 class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:20'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	self subclassResponsibility! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:31'!
decompiledSourceOf: aCompiledMethod
	
	^ aCompiledMethod decompile decompileString! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname
	
	^ self
		method: aStringOrSymbol
		of: aClassOrClassname 
		ifAbsent: [ self error: 'Method ', aStringOrSymbol asString, ' not found in class ', aClassOrClassname asString ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname ifAbsent: aBlock
	
	^ ((self classNamed: aClassOrClassname)
		lookupSelector: aStringOrSymbol asSymbol)
			ifNil: [ aBlock value ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/10/2011 08:58'!
print: aString to: aFilename
	"Create new file aFilename (or override) and write aString in it"
	
	^ aString squeakToUtf8 putOn: (FileStream forceNewFileNamed: aFilename)! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:33'!
randomMethod
	
	| class result |
	result := nil.
	[ result isNil ] whileTrue: [
		class := Smalltalk allClasses atRandom.
		class methodDict notEmpty
			ifTrue: [ result := class methodDict atRandom ]].
	^ result! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/8/2010 18:18'!
sourceOf: aCompiledMethod
	
	^ aCompiledMethod getSource asString! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
H =
	
	[ self halt ]! !

!OrcaOMeta2 methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:08'!
calledFunctions

	^ self inputConverter calledFunctions! !

!OrcaOMeta2 methodsFor: 'accessors' stamp: 'lw 4/6/2011 17:59'!
currentMethod
	
	^ currentMethod! !

!OrcaOMeta2 methodsFor: 'accessors' stamp: 'lw 4/6/2011 17:59'!
currentMethod: aCompiledMethod
	
	currentMethod := aCompiledMethod! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
fail =
	
	?[ false ]! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:50'!
finalize

	"Do nothing. Is overwritten in OrcaParser"
	self apply: #end.
! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:43'!
inputConverter
	^ inputConverter! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:43'!
inputConverter: anObject
	inputConverter := anObject! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
nil =
	
	anything:object ?[ object isNil ]
	-> [ object ]! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
node: aString
	
	^ self node: aString with: {}! !

!OrcaOMeta2 methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:08'!
referedClasses

	^ self inputConverter referedClasses! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:18'!
result
	^ result! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:18'!
result: anObject
	result := anObject! !

OrcaOMeta2 subclass: #OrcaParser
	instanceVariableNames: 'comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaParser commentStamp: 'ag 12/7/2010 23:09' prior: 0!
tokens
	Rules, that read one single element in squeal source-code. They all don't return an S2JNode, all are parsed to the
	according Squeak-object (e.g. Number, Symbol, String).
	Spaces before and after the tokens in the source-code are NOT read!!
	This is because in some cases, tokens must be read exactly, without emitting any spaces.
	The things returned from here must not be converted anymore, especially the <> operator is unnecessary.
	
complex elements
	Bigger parts of squeak source-code. Most return an S2JNode (using #node:with:), which contains further complex elements or object read
	by rules in the tokens-protocol. As convention, the nodes (except for helper-rules like primary and messagePattern) should be named like the rules.
	Each rule is supposed to read spaces before applying itself!! This way, rules from this protocol can be called serially, without
	reading spaces inbetween. However, spaces must be read between single tokens.
	
helpers
	Squeak-methods, that are called from within ometa-rules (or helper-rules).
	OMeta2 return-parts of the rules (righthand of ->) cannot contain multiple expressions (?)...
!

!OrcaParser class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:20'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ aString! !

!OrcaParser class methodsFor: 'parsing' stamp: 'ag 12/1/2010 11:04'!
parseNumber: aString
	"Parse aString to an instance of Number. Return nil, if parsing failed.
	Return nil also, if a number could be read, but not the whole String was processed.
	(Number readFrom: aString does not fail in that case)."
	
	| stream result |
	stream := ReadStream on: aString.
	result := Number readFrom: stream ifFail: [ nil ].
	stream atEnd ifFalse: [ result := nil ].
	^ result! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
array =
	
	$(
		(
			(spaces
				(((($# spaces)? array:array) -> [ array ])
				| nodeNamed(#symbolBaseForArray. 'symbol')
				| literalTokenBase
				):element
			) -> [ element ]
		)*:contents
	ch($))
	-> [ OrcaNode array: contents ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	spaces:comment binaryExpressionMessage:message
	-> [ OrcaNode named: 'binaryExpression' withoutNils:
		{ self extractCommentOf: comment. receiver }, message ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpressionMessage =
	// Not returning a node, but a simple collection
	
	binarySelector:selector
	(unaryExpression | primary):argument
	-> [ { selector. argument } ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
binarySelector =
	
	<$| | ((specialCharacter | $-) specialCharacter*)>:selector
	-> [ selector asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
block =
	
	<
	ch($[):comment1
		( (ch($:) spaces identifier:argumentName -> [ argumentName ])+:arguments ch($|)?:argumentSeparator )?
		temporaries?:temporaries
		statements?:statements
	ch($]):comment2
	>:originalCode
		
	// Forbidden (and not covered above): Arguments without separator bar, but with statements or temporaries
	?[ argumentSeparator notNil | arguments isNil | ((temporaries isNil or: [temporaries isEmpty]) & (statements isNil or: [statements isEmpty])) ]
	
	-> [ (OrcaNode named: 'block' withoutNils: { self extractCommentOf: comment1.
			OrcaNode named: 'arguments' withoutNilsNotEmpty: arguments.
			temporaries. statements.
			self extractCommentOf: comment2})
				originalCode: originalCode ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
braceExpression =
	
	ch(${):comment1
		(expression:expr ch($.) -> [ expr ] )*:expressions
		expression?:lastExpression
	ch($}):comment2
	
	-> [ self braceExpressionWith: expressions and: lastExpression ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
braceExpressionWith: expressions and: lastExpression
	
	^ OrcaNode named: 'braceExpression' withoutNils: 
		((expressions
			ifNotNil: [ expressions ]
			ifNil: [ {} ])
				asOrderedCollection
					add: lastExpression;
					yourself)! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
byteArray =
	// Byte array can only contain number-objects, don't pack them into nodes
	
	$# ch($[)
		(
			(spaces number:number [ number first ]:byte
			?[ (byte isInteger) & (byte <= 255) & (byte >= 0)] 
			) -> [ byte ]
		)*:contents
	ch($])
	-> [ OrcaNode byteArray: contents ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression =
	// Constructs a node, whose first node is the receiver and all further nodes are messages sent to that receiver
	
	(keywordExpression | binaryExpression | unaryExpression):mainExpression
	(
		ch($;)
		(
		((spaces binarySelector:selector (unaryExpression | primary):argument) -> [ OrcaNode binaryExpression: { selector. argument }])
		| (((spaces keyword:selector (binaryExpression | unaryExpression | primary):argument) -> 
				[ selector -> argument])+:selectorsAndArguments -> [ (self keywordMessage: selectorsAndArguments) name: 'keywordExpression' ])
		| (spaces unarySelector:selector -> [ OrcaNode unaryExpression: { selector } ])
		):cascade
		-> [ cascade ]
	)+:cascades
	-> [ self cascadedExpression: mainExpression with: cascades ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression: mainExpression with: cascades
	"The receiver of the mainExpression is the receiver of the whole cascade.
	It is the first node, the other nodes are the messages sent to that receiver"
	
	^ OrcaNode named: 'cascadedExpression' withoutNils: 
		((cascades
			ifNotNil: [ cascades ]
			ifNil: [ {} ])
				asOrderedCollection
					addFirst: mainExpression;
					addFirst: mainExpression removeFirst;
					yourself)! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
ch :c =
	// This reads a given element and all spaces before it and return the element
	// ch was supposed to mean character, but this can be applied to any object
	// Called like this: ch($=)
	
	spaces:comment exactly(c)
	-> [ comment ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
character =
	// The method in the super class is #char, don't confuse it!!
	
	( ($$ <anything>:value) | (seq('Character space') -> [ $  ]):value )
	-> [ OrcaNode character: value asCharacter ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
consumedBy: aBlock
	
	| result |
	result := super consumedBy: aBlock.
	result isString
		ifFalse: [ result := result separatedBy: '' ].
	^ result! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
doItSnippet =
	
	temporaries?:temps
	statements?:statements
	-> [ OrcaNode named: 'doItSnippet' withoutNils: { temps. statements } ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
expression =
	
	[ self startCollectingComments ]
	((spaces identifier:identifier (token(':=') | ch($_))) -> [ identifier ])*:variableNames 
	(cascadedExpression | keywordExpression | binaryExpression | unaryExpression | primary):expressionBody
	[ self stopCollectingComments ]:comment
	
	// First child is the expressionBody, further children are names of variables, that are assigned by this expression
	-> [ OrcaNode named: 'expression' withoutNils: ({ comment. expressionBody }, variableNames) ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
extractCommentOf: anOrderedCollection
	
	| result |
	true ifTrue: [^ nil]. "TODO skip all comments for now..."
	result := ''.
	anOrderedCollection do: [:item |
		item name = 'comment' ifTrue: [
			result := result , item first
			]].
	^ result isEmpty
		ifTrue: [nil]
		ifFalse: [self comment: result]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:50'!
finalize

	self apply: #spaces.
	super finalize! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
identifier =
	
	nodeNamed(#identifierBase. 'identifier')
|	(<$_ identifierBase>:decompiledIdentifier -> [ OrcaNode identifier: decompiledIdentifier ])! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
identifierBase =
	
	<letter letterOrDigit*>! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
initialize

	comments := Stack new.
	comments push: OrcaNodeNilObject new! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
keyword =
	
	<identifierBase $:>:selector
	-> [ selector asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	keywordExpressionMessage:message
	-> [OrcaNode keywordExpression: { receiver }, message ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpressionMessage =
	
	(((spaces keyword:selector ( binaryExpression | unaryExpression | primary ):argument) -> [ selector -> argument ])+:selectorAndArguments)
	-> [ self keywordMessage: selectorAndArguments ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
keywordMessage: selectorAndArgumentNames
	"selectorAndArgumentNames contains associations, the part of the selector and the argument name.
	Resulting OrcaNode will have the selector as symbol on the first position and the argumentNames at the other positions."
	
	| selector result |
	selector := ''.
	result := OrcaNode named: 'keyword'.
	selectorAndArgumentNames do: [ :each |
		selector := selector, each key.
		result add: each value].
	^ result
		addFirst: selector asSymbol;
		yourself! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
literalReservedWord =
	// Copy of reservedWord, with only the static values
	
	<(seq('true') | seq('false') | seq('nil'))>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ OrcaNode reservedWord: reservedWord ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
literalToken =
	// Anything, that can be evaluated to an object without context
	
	literalTokenBase
|	(($# spaces array:array) -> [ array ])! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
literalTokenBase =
	// Things, that can be evalutated to an object without any context (array-literals excluded)
	
	literalReservedWord
|	number
|	symbol
|	character
|	string
|	byteArray! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
messagePattern =
	
	((spaces keyword:selectorPart spaces identifier:argumentName) -> [ selectorPart -> argumentName ])+:selectorAndArgumentNames
	-> [ self keywordMessage: selectorAndArgumentNames ]
|	spaces binarySelector:selector spaces identifier:argumentName
	-> [ OrcaNode binaryExpression: { selector. argumentName } ]	
|	spaces unarySelector:unarySelector
	-> [ OrcaNode unaryExpression: { unarySelector } ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
method =
	// Only one temporaries-declaration allowed, but can be before or after primitive-declaration
	
	messagePattern:head
	[ self startCollectingComments ]
	temporaries?:temps
	primitiveDeclaration*:primitives
	temporaries?:secondTemps ( ?[secondTemps isNil]  | (?[temps isNil] [secondTemps]:temps) )
	[ self stopCollectingComments ]:comment
	statements?:statements
	
	-> [ OrcaNode named: 'method' withoutNils: { head. comment. temps }, primitives, { statements } ]! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
nodeNamed :aRule :aNodeName =
	// Helper rule to encapsulate the result of a rule into a node
	
	apply(aRule):result
	-> [ OrcaNode named: aNodeName asString with: { result } ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
number =
	// Called that way because of super-class-method
	// The string is parsed once, assuring, that
	
	<
		($-? digit+ $r)?
		$-?
		(digit | upper)+
		($. (digit | upper)+)?
		($e ($- | $+)? (digit | upper)+)?
	>:result
	?[ (self class parseNumber: result) notNil ]
	-> [ OrcaNode number: (self class parseNumber: result) ]
	
	// An alternative way to evaluate a number...
	| ($# <digit>:value) -> [ OrcaNode number: value asNumber ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
primary =
	// something, that evaluates to an object
	
	spaces (literalToken | reservedWord | identifier):result -> [ result ]
|	block
|	braceExpression
|	ch($() expression:expr ch($)) -> [ expr name: 'paranthesizedExpression'; yourself ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclaration =
	// Reads generally all primitive declarations, there are many of them
	// <primitive: 'a'> <primitive: 2> <primitive: 'a' module: 'b'> <version: 'ab'> <command> etc. etc.
	
	ch($<)
		(
			primitiveDeclarationRule(#keywordExpressionMessage)
			| (space primitiveDeclarationRule(#binaryExpressionMessage))
			| primitiveDeclarationRule(#unaryExpressionMessage)
		):message
	ch($>)
	-> [ OrcaNode named: 'primitiveDeclaration' with: { message } ]! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclarationRule :aRule =
	// Read everything, that is inside the <>-primitive-declaration; write it back on the input stream, followed
	// by input, that will not be matched. Apply the rule and read the fake-input. Return the result after applying the rule.
	
	// This allows things, that are forbidden, like <primive: a> (a is an identifier and has been accepted, because it's also a unarySelector
	
	<
		((
			<spaces (literalToken | unarySelector | binarySelector | keyword)>:input ?[ (input includesSubString: '>') not ]
		) -> [ input ])*
	>:primitiveInput
	[ self apply: aRule withArgs: primitiveInput, '^^' ]:result
	$^ $^
	-> [ result ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
reservedWord =
	
	<(seq('true') | seq('false') | seq('nil') | seq('self') | seq('super') | seq('thisContext') )>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ OrcaNode reservedWord: reservedWord ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
space =
	// A comment is equivalent to a space-character.
	// spaces are ignored (return nil), but comments result in a special node and are collected
	
	^space -> [ nil ]
	| (($" <(~$" anything)*>:result $") -> [ OrcaNode comment: result  ])! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
spaces =

	space*:whitespaces
	[ comments top addAllWhitoutNils: whitespaces ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
specialCharacter =
	
	$+ | $* | $/ | $\ | $~ | $< | $> | $= | $@ | $% | $& | $? | $!! | $` | $, | $! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
startCollectingComments

	comments push: (OrcaNode comments: {})! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
statements =
	
	(expression?:expr ch($.) -> [ expr ] )*:expressions
	(ch($^)?:returnSign expression:lastExpression ch($.)? ~expression)?
	-> [ self statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign
	"Build the contents of a statements-node from the information gathered while parsing 'statements'"
	
	"Rename the last expression, if it was a return-expression"
	lastExpression notNil & returnSign notNil ifTrue: [
		lastExpression name = 'expression'
			ifTrue: [ lastExpression name: 'returnExpression' ]
			ifFalse: [ self error: 'Unexpected node at return-sign: ', lastExpression printString ]].
	expressions
		add: lastExpression;
		reject: [ :each | each isNil ].
	^ OrcaNode named: 'statements' withoutNilsNotEmpty: expressions! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
stopCollectingComments
	
	^ comments pop ifEmpty: [ nil ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
string =
	// Called that way because of super-class-method
	
	$' ((~$' anything) | (($' $') -> [ $' ]))*:values $'
	-> [ OrcaNode string: (values asOrderedCollection inject: '' into: [ :a :b | a, b asString ]) ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
symbol =
	
	$# spaces (symbolBase | (string:stringSymbol -> [ stringSymbol first ])):value 
	-> [ OrcaNode symbol: value asSymbol ] 
	! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
symbolBase =
	// This does not return a symbol, just consume input. Inside of array, symbols can be represented without
	// the #-character
	
	binarySelector
|	<keyword+ identifierBase>
|	<keyword+>
|	identifierBase
|	($: | $. | ($[ $]) | ${ | $} | $] | $^):result -> [ result asSymbol ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
symbolBaseForArray =
	
	(~literalReservedWord
	(
		symbolBase
		| $[
	):result)
	-> [ result asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
temporaries =
	
	ch($|)
		(spaces identifier:identifier -> [ identifier ])*:variableNames
	ch($|)
	-> [ OrcaNode temporaries: variableNames ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpression =
	
	(unaryExpression | primary):receiver unaryExpressionMessage:message
	-> [ OrcaNode unaryExpression: {receiver. message} ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpressionMessage =
	
	spaces unarySelector:message
	-> [ message ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
unarySelector =
	
	identifierBase:selector ~$:
	-> [ selector asSymbol ]! !

!Boolean methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 12:00'!
definitionLine
	
	^ OrcaSqueakAstToJsAst jsKeywordFor: self asString! !

OrderedCollection subclass: #OrcaNode
	instanceVariableNames: 'name originalMethodName originalCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaNode commentStamp: 'ag 11/18/2010 13:09' prior: 0!
This is only a named collection, that is supposed to hold further collections or strings to form a tree!

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
arguments
	
	^ self arguments: {}! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:26'!
arguments: aCollection
	
	^ self named: 'arguments' with: aCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:59'!
array: aCollection

	^ self named: 'array' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
assignment: aCollection

	^ self named: 'assignment' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:15'!
binaryExpression: aCollection

	^ self named: 'binaryExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
block

	^ self block: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:26'!
block: aCollection

	^ self named: 'block' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 2/24/2011 13:34:07.814'!
braceExpression: aCollection

	^ self named: 'braceExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:37'!
byteArray: aCollection

	^ self named: 'byteArray' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:12'!
cascadedExpression: aCollection

	^ self named: 'cascadedExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:05'!
character: aCharacter

	aCharacter isCharacter ifFalse: [ self error: 'need a Character' ].
	^ self named: 'character' with: { aCharacter }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 4/5/2011 13:50:00.746'!
comment: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'comment' with: { aString }! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 4/5/2011 13:49:56.145'!
comments: anOrderedCollection

	^ self named: 'comments' with: anOrderedCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/25/2011 13:57'!
expression

	^ self named: 'expression' with: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
expression: aCollection

	^ self named: 'expression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
functionCall: aCollection
	"aCollection = { receiver. aMessageNameString. argumentsNode }"
	
	^ self named: 'functionCall' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
functionContent: aCollection

	^ self named: 'functionContent' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
functionDeclaration: aCollection

	^ self named: 'functionDeclaration' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
identifier: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'identifier' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'robi 3/22/2011 13:47'!
javaScriptMethod: aCollection
	"aCollection = { header. statements }"
	^ self named: 'javaScriptMethod' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:11'!
keywordExpression: aCollection

	^ self named: 'keywordExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'robi 3/22/2011 13:48'!
method: aCollection
	"aCollection = { header. statements }"
	^ self named: 'method' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'lw 2/22/2011 17:45'!
named: aString
	
	^ self named: aString with: {}! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:08'!
named: aString with: anOrderedCollection
	
	^ self new
		name: aString;
		addAll: anOrderedCollection;
		yourself! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:58'!
named: aString withoutNils: anOrderedCollection
	
	anOrderedCollection ifNil: [ ^ nil ].
	^ self
		named: aString
		with: (anOrderedCollection reject: [ :each | each isNil])! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/21/2010 19:15'!
named: aString withoutNilsNotEmpty: anOrderedCollection
	"Returns nil, if resulting node would be empty"
	
	| result |
	result := self named: aString withoutNils: anOrderedCollection.
	^ (result isNil or: [result isEmpty])
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 19:05'!
newExpression: aCollection

	^ self named: 'newExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
number: aNumber

	aNumber isNumber ifFalse: [ self error: 'need a Number' ].
	^ self named: 'number' with: { aNumber }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 2/24/2011 13:34:59.983'!
objectLiteral: aCollection

	^ self named: 'objectLiteral' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:51'!
paranthesizedExpression: aCollection

	^ self named: 'paranthesizedExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
reservedWord: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'reservedWord' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
returnExpression: aCollection

	^ self named: 'returnExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
slotAccess: aCollection

	^ self named: 'slotAccess' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
slotAssignment: aCollection

	^ self named: 'slotAssignment' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
statements

	^ self statements: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:24'!
statements: aCollection

	^ self named: 'statements' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
string: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'string' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:06'!
symbol: aSymbol

	aSymbol isSymbol ifFalse: [ self error: 'need a Symbol' ].
	^ self named: 'symbol' with: { aSymbol }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:32'!
temporaries
	
	^ self temporaries: {}! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:24'!
temporaries: aCollection
	
	^ self named: 'temporaries' with: aCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:48'!
unaryExpression: aCollection

	^ self named: 'unaryExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
variableDeclaration: aCollection

	^ self named: 'variableDeclaration' with: aCollection.! !

!OrcaNode methodsFor: 'comparing' stamp: 'ag 11/21/2010 19:19'!
= anObject
	
	^ super = anObject and: 
		[ name = anObject name ]! !

!OrcaNode methodsFor: 'adding' stamp: 'lw 4/5/2011 18:19'!
addAllWhitoutNils: aCollection
	
	aCollection do: [ :each | each ifNotNil: [self add: each]]
! !

!OrcaNode methodsFor: 'testing' stamp: 'ag 4/6/2011 15:52'!
includesChildNamed: aString
	
	^ self name = aString
		or: [ self anySatisfy: [ :node | 
			(node isKindOf: OrcaNode) and: [
			node includesChildNamed: aString ] ] ]! !

!OrcaNode methodsFor: 'testing' stamp: 'LT 12/1/2010 14:06:25.154'!
isArrayNode

	^ name = 'array' or: [name = 'byteArray' or: [name = 'braceExpression']]! !

!OrcaNode methodsFor: 'testing' stamp: 'LT 2/24/2011 13:54:04.347'!
isAssociationNode

	^ (self name = 'binaryExpression' and: [ (self at: 2) = #-> ])
	or: [ self name = 'expression' and: [ self size = 1 ] and: [ (self at: 1) isAssociationNode ]]! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name
	
	^ name! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name: aString
	
	name := aString! !

!OrcaNode methodsFor: 'accessing' stamp: 'lw 3/28/2011 17:40'!
originalCode
	^ originalCode! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/19/2011 19:17'!
originalCode: anObject
	"This is only set on 'block'-nodes. Never mind. :-)"
	
	originalCode := anObject! !

!OrcaNode methodsFor: 'printing' stamp: 'ag 11/19/2010 00:39'!
printString
	
	| result first |
	first := true.
	result := name, ' ('.
	self do: [ :each |
		first ifTrue: [first := false] ifFalse: [result := result, ', '].
		result := result,  each printString].
	^ result, ')'! !

ProtoObject subclass: #Js
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Clientside'!

!Js commentStamp: 'ag 4/5/2011 17:21' prior: 0!
This is a place, where programmers can define global shortcuts to access or execute common functions with long names.
The superclass of this class is set to nil to remove unnecessary methods.
Code inside this class should be short and is pure client-side code.!

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/5/2011 17:47'!
@ aString
	"This shortcuts calling the jQuery-$-function with one string as parameter.
	This is client-side code."
	
	^ (self Global perform: '$') value: aString! !

!Js class methodsFor: 'stubs' stamp: 'hk 4/5/2011 13:02'!
Document

	self clientSideResposibility.! !

!Js class methodsFor: 'stubs' stamp: 'hk 4/5/2011 13:03'!
Global

	self clientSideResposibility.! !

!Js class methodsFor: 'stubs' stamp: 'ag 4/12/2011 14:06'!
clearIntervals
	"This method clears all intervals, that has been set through code like 'Js Global setInterval value: aBlock value: aTime'.
	Only valid on the client-side. Such javascript-intervals sometimes don't disappear automatically when reloading a tab in the browser."
	
	self clientSideResposibility! !

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/5/2011 17:48'!
jit
	
	^ self Global perform: '$jit'! !

ProtoObject subclass: #OrcaFutureResult
	instanceVariableNames: 'computationCompleted result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaFutureResult class methodsFor: 'instance creation' stamp: 'lw 4/5/2011 12:25'!
newFrom: aBlock

	^ self new
		xxxStartComputation: aBlock;
		yourself! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:19'!
doesNotUnderstand: aMessage

	computationCompleted wait.
	self become: result.
	^ aMessage sendTo: result.! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:24'!
initialize

	computationCompleted := Semaphore new.! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:51'!
xxxStartComputation: aBlock

	[ result := aBlock value.
	computationCompleted signal ] fork! !

ProtoObject subclass: #OrcaRemoteObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaRemoteObject methodsFor: 'error handling' stamp: 'ag 4/6/2011 15:52'!
doesNotUnderstand: aMessage

	"Query on Client/Server and answer the result as a Future"
	"In order to reduce the selector footprint of this class, most selectors starts with xxx, alike the objectTracer"
	
	| aBlock aClient |
	aClient := nil.
	aBlock := [ aClient perform ].
	^ OrcaFutureResult newFrom: aBlock.! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 12:15'!
definitionLine
	
	^ (OrcaSqueakAstToJsAst escapeGlobal: self class name),
		'._newInstance()'! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
fillInstVars: codeLines guardedBy: alreadySerialized
	
	self class allInstVarNames withIndexDo: [ :varName :i | | varValue |
		varValue := self instVarAt: i. "This really works :D"
		varValue serializeToJavaScript: codeLines guardedBy: alreadySerialized.
		codeLines addLast: 
			(alreadySerialized at: self), '.', (OrcaSqueakAstToJsAst jsInstVarNameFor: varName), 
			'=', (alreadySerialized at: varValue) ].! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 16:22'!
serializeToJavaScript
	
	| code objectIds |
	code := OrderedCollection new.
	objectIds := Dictionary new.
	self serializeToJavaScript: code guardedBy: objectIds.
	code addLast: 'return ', (objectIds at: self).
	code := code separatedBy: ';'.
	^ code, ';' "function() { ... } is wrapped around this code on the server"! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 3/31/2011 10:02'!
serializeToJavaScript: codeLines guardedBy: alreadySerialized
	
	(alreadySerialized includesKey: self) ifFalse: [
		alreadySerialized at: self put: 'obj', (alreadySerialized size + 1) asString.
		codeLines addFirst:  'var ', (alreadySerialized at: self), '=', self definitionLine.
		self fillInstVars: codeLines guardedBy: alreadySerialized ].! !

!HTTPConnection methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 17:50'!
interact
	"We refine this to bring up the debugger when an error occurs inside the Swazoo."



	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[[	[ [true] whileTrue: 
				[self getAndDispatchMessages.
				Processor yield] 
			]
			on: Error
			do: [:ex | 
				
				((Smalltalk at: #OrcaDebugSwazooInternalErrors ifAbsent: [nil]) == true)
					ifTrue: [ (ex class = SwazooStreamNoDataError) | (ex class = SpSocketError) ifFalse: [ self halt ] ]
				] ]
		ifCurtailed: [ 
			(Delay forMilliseconds: 50) wait. "to finish sending, if any"
			self isOpen ifTrue: [self close]] ].
	self server isMultiThreading 
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchMessages handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:05'!
classesWithDirtyCategories
	
	^ self ourClasses select: [ :class |
		(class name endsWith: 'Test') not & (class name endsWith: 'Tests') not &
		(class organization categories includes: 'as yet unclassified' asSymbol) ]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:02'!
classesWithEmptyCategories
	
	^ (self ourClasses select: [ :class |
			class organization categories anySatisfy: [ :category | class organization isEmptyCategoryNamed: category ] ]),
		self ourEmptyClassCategories! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:02'!
countCategoriesOf: aClass
	
	^ (aClass organization categories
		reject: [ :cat | cat == 'no messages' asSymbol ])
			size! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 17:50'!
debugSwazooErrors: aBoolean
	
	^ Smalltalk
		at: #OrcaDebugSwazooInternalErrors
		put: aBoolean! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 3/28/2011 18:01'!
grep: aString
	"Search the source-code of our packages and return a collection of all CompiledMethods, that contain aString in their source-code."
	
	| methods |
	methods := OrderedCollection new.
	self ourClasses do: [ :class |
		class methodDict values do: [ :compiledMethod |
			(compiledMethod value getSource asString includesSubString: aString)
					ifTrue: [ methods add: compiledMethod ]]].
	^ methods! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:03'!
ourClasses
	
	^ self ourPackage classesAndMetaClasses! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:03'!
ourEmptyClassCategories
	
	^ self ourPackage systemCategories select: [ :cat |
		SystemOrganization isEmptyCategoryNamed: cat]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/11/2011 11:46:52.242'!
ourPackage
	
	^ PackageOrganizer default
		packageNamed: 'Orca' asSymbol
		ifAbsent: [ self error: 'Our package Orca could not be found...' ]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:03'!
removeEmptyCategories
	"Removes method-categories and class-categories. Return, how many categories have been removed."
	
	| removedCategories |
	removedCategories := 0.
	self ourClasses do: [ :class | | categoriesBefore |
		categoriesBefore := self countCategoriesOf: class.
		class organization removeEmptyCategories.
		removedCategories := removedCategories + categoriesBefore - (self countCategoriesOf: class) ].
	self ourEmptyClassCategories do: [ :cat |
		SystemOrganization removeCategory: cat.
		removedCategories := removedCategories + 1].
	^ removedCategories! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 14:07'!
renamePackagesNamed: oldName to: newName
	
	PackageOrganizer default packages do: [ :pk | | classes |
		pk classesAndMetaClasses do: [ :cl |
			cl organization categories do: [ :cat |
				(cat = oldName asSymbol)
					ifTrue: [ cl organization
						renameCategory: oldName
						toBe: newName ] ] ]].! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 14:07'!
renamePrefixFrom: oldPrefix to: newPrefix

	PackageOrganizer default packages do: [ :pk | | classes |
		pk classes do: [ :cl | | n |
			n := cl name.
			(n beginsWith: oldPrefix) ifTrue: [
				(RenameClassRefactoring
					rename: cl
					to: newPrefix, (n copyFrom: (oldPrefix size + 1) to: n size))
						execute ] ]].! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'lw 4/6/2011 18:08'!
withinEachMethodOf: aClass replace: aRegularExpressionString with: aString ignoreCase: aBool

	| regexRefactoring |
	regexRefactoring := ORRegexRefactoring new
		replace: aRegularExpressionString with: aString ignoreCase: aBool;
		yourself.

	aClass methodDict do: [ :aCompiledMethod | 
			aClass compile: (regexRefactoring execute: aCompiledMethod getSource) ]! !

!String methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst stringFunction asString, '(', (OrcaAbstractPrinter asJSString: self), ')'! !

!String methodsFor: '*Orca-Core' stamp: 'sw 3/30/2011 14:24'!
jsEval

	^ Js Global eval value: self! !

!MIMEDocument class methodsFor: '*Orca-Core' stamp: 'hk 1/26/2011 10:40'!
extendedMIMEdatabase
	| d |
	(d := self defaultMIMEdatabase)
		 at: 'js' put: 'text/javascript';
		 at: 'hqx' put: 'application/mac-binhex40';
		 at: 'cpt' put: 'application/mac-compactpro';
		 at: 'pdf' put: 'application/pdf';
		 at: 'ps' put: 'application/postscript';
		 at: 'ai' put: 'application/postscript';
		 at: 'eps' put: 'application/postscript';
		 at: 'rtf' put: 'text/rtf';
		 at: 'ics' put: 'text/calendar';
		 at: 'bin' put: 'application/octet-stream';
		 at: 'dms' put: 'application/octet-stream';
		 at: 'lha' put: 'application/octet-stream';
		 at: 'lzh' put: 'application/octet-stream';
		 at: 'exe' put: 'application/octet-stream';
		 at: 'class' put: 'application/octet-stream';
		 at: 'zip' put: 'application/zip';
		 at: 'gtar' put: 'application/x-gtar';
		 at: 'swf' put: 'application/x-shockwave-flash';
		 at: 'sit' put: 'application/x-stuffit';
		 at: 'tar' put: 'application/x-tar';
		 at: 'au' put: 'audio/basic';
		 at: 'snd' put: 'audio/basic';
		 at: 'mid' put: 'audio/midi';
		 at: 'midi' put: 'audio/midi';
		 at: 'mpga' put: 'audio/mpeg';
		 at: 'mp2' put: 'audio/mpeg';
		 at: 'mp3' put: 'audio/mpeg';
		 at: 'aiff' put: 'audio/x-aiff';
		 at: 'aif' put: 'audio/x-aiff';
		 at: 'aifc' put: 'audio/x-aiff';
		 at: 'rm' put: 'audio/x-pn-realaudio';
		 at: 'ram' put: 'audio/x-pn-realaudio';
		 at: 'rpm' put: 'audio/x-pn-realaudio-plugin';
		 at: 'ra' put: 'audio/x-realaudio';
		 at: 'wav' put: 'audio/x-wav';
		 at: 'css' put: 'text/css';
		 at: 'mpeg' put: 'video/mpeg';
		 at: 'mpg' put: 'video/mpeg';
		 at: 'mpe' put: 'video/mpeg';
		 at: 'qt' put: 'video/quicktime';
		 at: 'mov' put: 'video/quicktime';
		 at: 'avi' put: 'video/x-msvideo';
		 at: 'movie' put: 'video/x-sgi-movie'.
	^ d! !

SwazooResource subclass: #OrcaResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaResource subclass: #OrcaBasicApp
	instanceVariableNames: 'clients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaBasicApp class
	instanceVariableNames: 'instance'!

OrcaBasicApp class
	instanceVariableNames: 'instance'!

OrcaBasicApp subclass: #OrcaApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'ag 1/25/2011 15:49'!
allRequiredClasses
	"Return the requiredClasses together with all their superclasses. Only for this class, not the superclasses."
	
	| allClasses |
	allClasses := OrderedCollection new.
	self requiredClasses do: [ :eachClass |
		eachClass allSuperclasses reverseDo: [ :superClass |
			(allClasses includes: superClass) ifFalse: [ allClasses add: superClass ]].
		(allClasses includes: eachClass) ifFalse: [ allClasses add: eachClass ]].
	^ allClasses! !

!OrcaApp class methodsFor: 'rendering-configuration' stamp: 'ag 4/6/2011 15:52'!
codeFactory
	
	^ OrcaPrettyCodeFactory.! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'SW 1/31/2011 08:35'!
codeForAllClasses

	^ self codeFactory codeForClasses: self collectRequiredClasses! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'ag 4/6/2011 15:52'!
collectPostloadScripts
	"Collect #requiredClasses from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass postloadScripts ]
		upTo: OrcaApp! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'ag 4/6/2011 15:52'!
collectRequiredClasses
	"Collect #requiredClasses from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass allRequiredClasses ]
		upTo: OrcaApp! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'LT 4/13/2011 15:27:33.979'!
postloadScripts
	
	^ #( 'js/perform.js' 'js/boxing.js' 'js/bootstrap.js' 'js/remoteObjects.js'
	
	'js/primitives/Array.js' 'js/primitives/BlockClosure.js' 'js/primitives/Exception.js' 'js/primitives/Float.js' 'js/primitives/Js.js' 'js/primitives/Number.js' 'js/primitives/Object.js'
	'js/primitives/OrcaWidget.js' 'js/primitives/Point.js' 'js/primitives/ProtoObject.js' 'js/primitives/String.js' )! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:04'!
recompileRequiredClasses
	"This causes all required classes of THIS web-app (none of the super-classes) to be invalidated in the code-factory-cache."
	
	^ self allRequiredClasses do: [ :each | each compileAll ]! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'sw 4/7/2011 16:35'!
requiredClasses
	"Return any array of classes, that must be compiled to JS-code to enable this application.
	Classes added by subclasses must also return the classes in this implementation, as these clases are required by the js-bootstrapping-library."
	
	^ { Js. BlockClosure. Float. True. False. Error. ByteString. OrderedCollection. Character. Array. UndefinedObject. Point. WriteStream . Message  . Association }! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'ag 4/11/2011 19:45'!
scripts
	"= preload scripts"
	
	^ #( 'js/classes.js' )! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 16:35'!
startupClass
	"Return something other than nil to call the static method #start on the class returned here."
	
	^ nil! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'ag 4/12/2011 14:29'!
startupFunction
	
	^ OrcaSqueakAstToJsAst escapeGlobal: 'startupOrca'! !

!OrcaApp class methodsFor: 'rendering-javascript' stamp: 'ag 4/12/2011 14:29'!
startupVariable
	
	^ OrcaSqueakAstToJsAst escapeGlobal: 'app'! !

!OrcaApp class methodsFor: 'rendering-css' stamp: 'SW 2/21/2011 15:13'!
styles
	
	^ #( 'styles/default.css' )! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:05'!
writeClassesTo: aDirectoryName
	"Write all classes required by this WebApp (together with their superclasses) to files named '<classname>.txt' in the given directory."
	
	self codeForAllClasses.
	self collectRequiredClasses withIndexDo: [ :cl :index |
		self codeFactory
			printCodeFor: cl
			to: aDirectoryName, index asString, '-', cl name, '.txt' ].! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:05'!
writeClassesToFile: aFileName
	"Write all classes required by this WebApp (together with their superclasses) to files named '<classname>.txt' in the given directory."
	
	^ self codeFactory
		printCodeFor: self collectRequiredClasses
		to: aFileName! !

!OrcaApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:06'!
writeSiteTo: aLocalFilename
	"Write the website, that would be sent to a client, to a new file named aLocalFilename (overwrites existing)."
	
	^ self codeFactory converterClass
		print: self new render
		to: aLocalFilename! !

!OrcaApp methodsFor: 'hooks' stamp: 'ag 4/12/2011 14:30'!
onloadCode
	
	^ super onloadCode, self class startupVariable, '=', self class startupFunction, '();'! !

!OrcaApp methodsFor: 'rendering' stamp: 'ag 4/6/2011 15:52'!
renderJavascript
	"Render the external javascript resources and the link to the compiled classes code."
	
	^
	super renderJavascript,
	(self renderScriptTag: OrcaClassesHandler pathSuffix),
	(self renderScriptTags: self class collectPostloadScripts),
	self renderStartupCall, String cr! !

!OrcaApp methodsFor: 'rendering' stamp: 'ag 4/12/2011 14:34'!
renderStartupCall
	
	^ 
	'<script type="text/javascript">',
	self class startupFunction, ' = function() {', (self class startupClass
		ifNotNil: [ 'return ', (self class codeFactory codeFor: (self class startupClass name, ' start')) ]
		ifNil: [ '' ]),
			'}',
	'</script>'! !

OrcaApp subclass: #OrcaIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaIndex class methodsFor: 'configuration' stamp: 'ag 1/12/2011 18:21'!
title

	^ 'Index'! !

!OrcaIndex class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath

	^ 'index/'! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:46'!
findConflicts

	| conflicts |
	conflicts := Set new.
	OrcaApp allSubclassesDo: [ :each | |group|
		group := OrcaApp allSubclasses select: [ :every |
			every uniquePath notNil
			and: [ every uniquePath = each uniquePath ]].
		(group size > 1) ifTrue: [ conflicts add: group asString ]
	].

	^ conflicts! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:50'!
linkFor: anApplication

	^ '<a href="/', anApplication uniquePath , '" title="', anApplication title ,'">', anApplication title , '</a>'
	    
	
	! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:08'!
renderAmbigousApplications

	^
		'<h3>Conflicting applications</h3>',
		self renderConflicts! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:15'!
renderConflicts

	| list | 
	list := '<ul>'.
	
	self findConflicts do: [ :each |
		list := list, '<li>', each asString, '</li>' ]..
	
	^ list, '</ul>'.
	! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 10:56'!
renderContent

	^ 
		self renderHeading,
		self renderInstalledApplications,
		self renderIncompleteApplications,
		self renderAmbigousApplications! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 10:55'!
renderHeading

	^ '<h1>Orca Framework</h1>'! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:58'!
renderIncompleteApplications
	
	| list |
	list := '<ul>'.
	(OrcaApp allSubclasses reject: [ :each | each isComplete ]) do: [ :each |
		list := list , '<li>', each name , '</li>' ].
	list := list, '</ul>'.
	
	^ '<h3>Incomplete applications</h3>', list! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 10:59'!
renderInstalledApplications

	^
		'<h3>Installed applications:</h3>',
		self renderLinks! !

!OrcaIndex methodsFor: 'rendering' stamp: 'hk 4/8/2011 11:59'!
renderLinks

	| result |
	result := '<ul>'.
	OrcaBasicApp allSubclasses
		select: [ :each | each isComplete xor: (each = self class) ]
		thenDo: [ :each | result := result, '<li>', (self linkFor: each) , '</li>' ].
	
	^ result, '</ul>'! !

!OrcaBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 17:00'!
collect: aBlock upTo: topSuperClass
	"Evaluate aBlock with all relevant superclasses as single argument and collect the resulting collections."
	
	| class result superClasses |
	result := OrderedCollection new.
	superClasses := topSuperClass allSuperclasses.
	class := self.
	[ superClasses includes: class ]
		whileFalse: [
			result addAllFirst: (aBlock value: class).
			class := class superclass ].
	^ result
		inject: OrderedCollection new
		into: [ :all :cl | (all includes: cl) ifFalse: [ all add: cl ]. all ]! !

!OrcaBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 4/6/2011 15:52'!
collectScripts
	"Collect #scripts from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass scripts ]
		upTo: OrcaBasicApp! !

!OrcaBasicApp class methodsFor: 'rendering-css' stamp: 'ag 4/6/2011 15:52'!
collectStyles
	"Collect #styles from all superclasses"
	
	^ (self
		collect: [ :eachClass | eachClass styles ]
		upTo: OrcaBasicApp)! !

!OrcaBasicApp class methodsFor: 'configuration' stamp: 'HK 1/10/2011 16:30'!
htmlDocumentType

	^ '<!!DOCTYPE html>'.! !

!OrcaBasicApp class methodsFor: 'singleton' stamp: 'HK 1/12/2011 10:43'!
instance
	
	instance ifNil: [ instance := self new ].
	^ instance.! !

!OrcaBasicApp class methodsFor: 'hooks' stamp: 'hk 4/8/2011 11:58'!
isComplete

	^ self uniquePath notNil and: [ self title notNil ]! !

!OrcaBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 4/11/2011 21:28'!
scripts
	
	^ #( 'js/helpers.js' 'js/console.js' 'js/communication.js' 'js/server.js' )! !

!OrcaBasicApp class methodsFor: 'rendering-css' stamp: 'hk 1/26/2011 12:29'!
styles

	^ #()! !

!OrcaBasicApp class methodsFor: 'configuration' stamp: 'ag 1/24/2011 17:19'!
title
	"Must be implemented by non-abstract subclasses"
	
	^ nil! !

!OrcaBasicApp class methodsFor: 'configuration' stamp: 'ag 1/24/2011 17:20'!
uniquePath
	"Must be implemented by non-abstract subclasses"
	
	^ nil! !

!OrcaBasicApp methodsFor: 'dispatching' stamp: 'ag 1/25/2011 13:41'!
addClient: aClient

	self clients add: aClient.
	aClient application: self.! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'SW 1/18/2011 11:26'!
answerNoContent

	^ self answerWithStatusCode: 204! !

!OrcaBasicApp methodsFor: 'dispatching' stamp: 'ag 4/6/2011 15:52'!
answerTo: aRequest
	
	| handlerClass |
	handlerClass := OrcaClientHandler subclasses
		detect: [ :eachClass | eachClass canHandle: aRequest ]
		ifNone: [ nil ].
	^ handlerClass
		ifNotNil: [ (handlerClass newFor: self) answerTo: aRequest ]
		ifNil: [ self answerContent: self render withType: 'text/html' ]! !

!OrcaBasicApp methodsFor: 'accessing' stamp: 'hk 3/28/2011 17:12'!
clients
	clients ifNil: [ clients := OrderedCollection new ].
	^ clients! !

!OrcaBasicApp methodsFor: 'accessing' stamp: 'HK 1/12/2011 10:49'!
clients: anObject
	clients := anObject! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 4/6/2011 15:52'!
fullUrlFor: aUrl
	^ (aUrl startsWith: 'http')
		ifTrue: [ aUrl ]
		ifFalse: [ OrcaFileRequestHandler pathSuffix , '/' , aUrl ]! !

!OrcaBasicApp methodsFor: 'initialization' stamp: 'LT 4/12/2011 13:38:17.151'!
initialize

	super initialize.
	self clients: OrderedCollection new.! !

!OrcaBasicApp methodsFor: 'hooks' stamp: 'ag 4/11/2011 19:52'!
onloadCode
	
	^ 'st.communication.connect();'! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'HK 1/12/2011 15:13'!
removeAllClients

	self clients
		select: [ :each | each isWebSocketClient ]
		thenDo: [ :each | each close ].
	self clients: OrderedCollection new.! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 11:47'!
removeClient: aClient
	
	self clients remove: aClient ifAbsent: [ ].! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/13/2011 20:18'!
render
	
	^ self class htmlDocumentType,
		'<html>',
			self renderHeader, 
			self cr, 
			self renderBody,
		'</html>'! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/24/2011 16:45'!
renderBody

	^'<body onload="', self onloadCode, '" onunload="">',
		self renderContent,
	   '</body>'.! !

!OrcaBasicApp methodsFor: 'hooks' stamp: 'ag 1/24/2011 14:31'!
renderContent 
	"Subclasses return custom html-content"
	
	^ ''! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'hk 1/26/2011 12:20'!
renderHeader

	^ '<head>',
		self renderTitle,
		self renderJavascript,
		self renderStyles,
	'</head>'.! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'srv 1/24/2011 14:08'!
renderJavascript
	"Render the external javascript resources."
	
	^ self renderScriptTags: self class collectScripts! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 09:56'!
renderScriptTag: aFilename
	
	^ '<script type="text/javascript" src="', aFilename, '"></script>', String cr! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 14:56'!
renderScriptTags: aCollectionOfFilenames
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames of the js-files. The path (url) will be prepended."
	
	^ self
		renderTags: aCollectionOfFilenames
		collectingFrom: [ :url | self renderScriptTag: url]! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'hk 1/26/2011 12:27'!
renderStyleTag: aFilename
	
	^ '<link href="', aFilename, '" rel="stylesheet" type="text/css" />', String cr! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 15:07'!
renderStyles
	"Render the external javascript resources."
	
	^ self
		renderTags: self class collectStyles
		collectingFrom: [ :url | self renderStyleTag: url]! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 14:55'!
renderTags: aCollectionOfFilenames collectingFrom: aBlock
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames. The path (url) will be prepended."
	
	^ aCollectionOfFilenames
		inject: ''
		into: [ :string :eachUrl | 
			string, (aBlock value: (self fullUrlFor: eachUrl)) ]! !

!OrcaBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 09:57'!
renderTitle
	
	^'<title>',
	 self class title,
	'</title>',
	String cr! !

!OrcaBasicApp methodsFor: 'hooks' stamp: 'LT 4/12/2011 13:38:32.342'!
reset 
	
	self clients: OrderedCollection new.! !

!OrcaBasicApp methodsFor: 'sending content' stamp: 'LT 4/12/2011 12:20:56.345'!
sendCode: code
	
	| jsCode |
	jsCode := self translateSqueakCode: code.
	self clients do: [ :each | each receive: jsCode ].! !

!OrcaBasicApp methodsFor: 'sending content' stamp: 'LT 4/12/2011 12:21:33.336'!
translateSqueakCode: code

	| jsCode |
	[ jsCode := OrcaPrettyPrinter convertDoIt: code ]
		on: Exception
		do: [ :err | jsCode := 'alert("Could not parse squeak-code: ', Character cr asString, code,
			Character cr asString, 'Error was: ', err asString, '");' ].
	^ jsCode! !

!OrcaBasicApp methodsFor: 'accessing' stamp: 'LT 4/12/2011 12:23:16.385'!
webSocketClients
	
	^ self clients select: [ :each | each isWebSocketClient ]! !

OrcaResource subclass: #OrcaClientHandler
	instanceVariableNames: 'application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaClientHandler subclass: #OrcaClassesHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaClassesHandler class methodsFor: 'request handling' stamp: 'ag 1/26/2011 09:13'!
pathSuffix
	
	^ 'classes'! !

!OrcaClassesHandler methodsFor: 'request handling' stamp: 'ag 1/26/2011 09:19'!
answerTo: aRequest
	
	^ self answerCode: self application class codeForAllClasses! !

!OrcaClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 16:29'!
canHandle: aRequest
	
	| segments |
	segments := self pathSegments: aRequest.
	^ segments size >= 2 and: [ segments second = self pathSuffix ]! !

!OrcaClientHandler class methodsFor: 'instance creation' stamp: 'HK 1/11/2011 16:33'!
newFor: anApplication

	^ self new application: anApplication.! !

!OrcaClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:31'!
pathSegments: aRequest
	
	^ (aRequest requestLine requestURI identifier) findTokens: '/'! !

!OrcaClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:07'!
pathSuffix
	
	self subclassResponsibility! !

!OrcaClientHandler methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:08'!
answerTo: aRequest
	
	self subclassResponsibility! !

!OrcaClientHandler methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:33'!
application
	^ application! !

!OrcaClientHandler methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:33'!
application: anObject
	application := anObject! !

!OrcaClientHandler methodsFor: 'identification' stamp: 'LT 4/7/2011 16:22:52.191'!
extractIdentifierFrom: aRequest

	| matches |
	matches := 'id=([0-9]*)&?' asRegex submatchesIn: aRequest requestLine requestURI asString.
	^ matches first first asInteger! !

!OrcaClientHandler methodsFor: 'request handling' stamp: 'LT 4/12/2011 19:37:40.236'!
findClient: aRequest

	| identifier |
	identifier := self extractIdentifierFrom: aRequest.
	^ self getOrCreateClientById: identifier andAddress: aRequest ip.! !

!OrcaClientHandler methodsFor: 'identification' stamp: 'LT 4/12/2011 19:37:40.223'!
getOrCreateClientById: anIdentifier andAddress: anIpAddress

	^ self application clients
		detect: [ :uniqueClient |
			uniqueClient isCometClient and:
				[ uniqueClient identifier = anIdentifier and: [ uniqueClient ipAddress = anIpAddress]]]
		ifNone: [ | client | 
			client := (OrcaCometClient from: anIpAddress identifiedBy: anIdentifier).
			self application addClient: client.
			^client ]! !

!OrcaClientHandler methodsFor: 'identification' stamp: 'LT 4/7/2011 15:33:52.869'!
includesIdentifier: aRequest

	^ aRequest requestLine requestURI asString includesSubString: '?id='.! !

OrcaClientHandler subclass: #OrcaCometHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaCometHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix
	
	^ 'xhr'! !

!OrcaCometHandler methodsFor: 'responding' stamp: 'HK 1/3/2011 20:30'!
answerNewIdentifier: anIdentifier

	^ self
		answerWithStatusCode: 202
		andContent: anIdentifier.! !

!OrcaCometHandler methodsFor: 'request handling' stamp: 'LT 4/12/2011 13:53:14.767'!
answerTo: aRequest
	
	| client |
	client := self findClient: aRequest.
	^ client
		ifNil: [ self registerNewClient ]
		ifNotNil: 
			[ (client isCometClient and: [aRequest isGet])
				ifTrue: [client update]].! !

OrcaClientHandler subclass: #OrcaFileRequestHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix
	
	^ 'file'! !

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/6/2011 18:01'!
rootDirectory: aDirectoryName
	"Configure the root-directory, this mini-server is serving. Only required once."
	
	| dir |
	dir := FileDirectory on: aDirectoryName.
	dir exists ifFalse: [ self error: 'Directory does not exist: ', aDirectoryName ].
	^ Smalltalk
		at: #OrcaFileRootDirectory
		put: dir! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'hk 3/29/2011 11:23'!
answerTo: aRequest
	
	| file |
	file := self extractFile: aRequest.
	^ file
		ifNotNil: [
			file binary.
			"images require binary mode. plain text works regardless of encoding"
			self application
				answerCode: file contents
				withType: (self mimeTypeOf: file) ]
		ifNil: [
			self application
				answerWithStatusCode: 404
				andContent: 'Requested file not found' ]! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'ag 4/6/2011 18:01'!
extractFile: aRequest
	
	| segments fileRoot |
	segments := self class pathSegments: aRequest.
	segments removeFirst. "The app-name"
	segments removeFirst. "The /file -suffix"
	fileRoot := Smalltalk
		at: #OrcaFileRootDirectory
		ifAbsent: [ 
			| possiblePathName possibleDirectory |

			possiblePathName := FileDirectory default pathName, 
								FileDirectory pathNameDelimiter asString, 'orca'.
			
			possibleDirectory := FileDirectory on: possiblePathName.
			
			possibleDirectory exists 
				ifTrue: [ possibleDirectory ]
				ifFalse: [
					self error: 
'The root-directory name has not been configurd!! 
Execute: OrcaFileRequestHandler rootDirectory: "/absolute/path/to/root/"
or: OrcaInstaller setupFileDirectory'.
				].
			 ].
	^ fileRoot oldFileOrNoneNamed: (segments separatedBy: FileDirectory pathNameDelimiter asString)! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'hk 3/29/2011 11:21'!
mimeTypeOf: aFile

	^ MIMEDocument guessTypeFromName: aFile name! !

OrcaClientHandler subclass: #OrcaMethodInvocationHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaMethodInvocationHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix 
	
	^ 'mi'! !

!OrcaMethodInvocationHandler methodsFor: 'request handling' stamp: 'ag 3/1/2011 17:28'!
answerTo: aRequest

	| code arguments result |
	aRequest ensureFullRead.
	code := (aRequest postDataAt: 'code') value.
	arguments := self retrieveArgumentsFrom: aRequest.
	result := (Compiler evaluate: code) valueWithArguments: arguments asArray.
	^ self application answerContent: result serializeToJavaScript withType: 'text/javascript'! !

!OrcaMethodInvocationHandler methodsFor: 'request handling' stamp: 'LT 2/21/2011 14:47:33.929'!
retrieveArgumentsFrom: aRequest

	| arguments currentArg |
	arguments := OrderedCollection new.
	currentArg := 0.
	[ currentArg isNil ]
		whileFalse: [ 
			| key arg |
			key := 'arg' , currentArg asString.
			arg := (aRequest postDataAt: key) value.
			arg
				ifNil: [ currentArg := nil ]
				ifNotNil: [ 
					arguments add: arg.
					currentArg := currentArg + 1 ] ].
	^ arguments! !

OrcaClientHandler subclass: #OrcaRemoteMessageHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaRemoteMessageHandler class methodsFor: 'request handling' stamp: 'LT 4/7/2011 15:18:38.887'!
pathSuffix 
	
	^ 'send'! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'LT 4/13/2011 15:49:18.394'!
answerTo: aRequest

	| className client remoteID |
	aRequest ensureFullRead.
	className := (aRequest postDataAt: 'newObjectOfClassNamed') value.
	client := self findClient: aRequest.
	remoteID := client addReachableObjectFor: (Smalltalk classNamed: className) new.
	^ self application answerContent: remoteID asString withType: 'text/plain'! !

OrcaClientHandler subclass: #OrcaWebSocketHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaWebSocketHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 16:29'!
canHandle: aRequest
	"This class has an own implementation of this. That's why the abstract method pathSuffix is not implemented."
	
	^ aRequest isWebSocketRequest! !

!OrcaWebSocketHandler class methodsFor: 'configuration' stamp: 'HK 1/4/2011 12:17'!
millisecondsToWaitForHandshake

	^ 200.! !

!OrcaWebSocketHandler class methodsFor: 'client handling' stamp: 'hk 4/5/2011 11:15'!
pathSuffix
	
	^ 'ws'! !

!OrcaWebSocketHandler methodsFor: 'request handling' stamp: 'SW 1/17/2011 12:23'!
answerTo: aRequest

	| webSocket |
	webSocket := aRequest webSocket.
		
	(self isNew: webSocket ) ifTrue: 
		[ self setupClientWith: webSocket ].
	^ webSocket answerTo: aRequest.! !

!OrcaWebSocketHandler methodsFor: 'convenience' stamp: 'HK 1/12/2011 16:25'!
isNew: aWebSocket

	| registeredSockets |
	registeredSockets := self application webSocketClients collect:
		[ :each | each socket ].
	
	^ (registeredSockets includes: aWebSocket) not.! !

!OrcaWebSocketHandler methodsFor: 'convenience' stamp: 'ag 4/6/2011 15:52'!
setupClientWith: aWebSocket

	| client |
	client := (OrcaWebSocketClient newWithSocket: aWebSocket).
	self application addClient: client.
	"self updateDelayed: client."! !

OrcaResource subclass: #OrcaDispatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 13:57'!
defaultPort

	^ 8887! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'lw 2/25/2011 14:07'!
forcedRestart
	"Why is this even necessary?" "Brought to you by UGLYFIX!! :-)"
	
	Transcript show: 'Server restart.';cr.
	SwazooServer initSingleton.
	SwazooServer allInstancesDo: [ :swa | 
		swa stop; initSites].
	HTTPServer allInstancesDo: [ :http |
		http stop; initialize].
	HTTPConnection allInstancesDo: [ :conn |
		conn close].
	SwazooServer singleton start.
	Smalltalk garbageCollect.
	self start.! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
install
	
	^ self installOn: self defaultPort! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 14:16'!
installOn: aPort
	
	^ (SwazooSite
		newNamed: self name)
		host: '*'
		ip: '*'
		port: aPort;
		addResource: (self uriPattern: '/');
		yourself! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
isInstalled
	
	^ self site notNil! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 14:16'!
site
	
	^ SwazooServer singleton siteNamed: self name! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
start
	
	self isInstalled ifFalse: [ self install ].
	self site start.! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:13'!
uninstall
	
	self isInstalled ifTrue: [
		self removeAllClients.
		self resetContent.
		SwazooServer singleton removeSite: self site ].! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'LT 4/11/2011 13:20:20.781'!
answerTo: aRequest 
	
	| application |
	(self hasToBeForwarded: aRequest ) 
		ifTrue: [ ^ self forward: aRequest ].		
	application := self findApplicationNamed: (self identifierOf: aRequest).
	application
		ifNil: [  ^ self redirectTo: '/', OrcaIndex uniquePath ]
		ifNotNil: [ ^ application instance answerTo: aRequest ].! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/6/2011 15:52'!
findApplicationNamed: aName

	aName ifNil: [ ^ nil ].
	^ OrcaBasicApp allSubclasses
		detect: [ :each | each uniquePath = aName ]
		ifNone: [ nil ]. ! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 12:27'!
forward: aRequest

	| url |
	url:= self identifierOf: aRequest.
	^ self redirectTo: '/', url! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 12:19'!
hasToBeForwarded: aRequest
	
	| url |
	url := aRequest requestLine requestURI identifier.
	^ ( url findTokens: '/') size = 1 and: [ (url endsWith: '/') not ]
	
	
! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 11:54'!
identifierOf: aRequest
	
	((aRequest requestLine requestURI identifier) findTokens: '/') size = 0 
		ifTrue: [ ^ nil  ]
		ifFalse: [ ^ ((aRequest requestLine requestURI identifier) findTokens: '/') first, '/' ].
	
	
! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'HK 1/4/2011 15:45'!
redirectTo: aPath

	^HTTPResponse new
		statusCode: 302;
		entity: '';
		location: aPath;
		yourself.! !

!OrcaResource methodsFor: 'responding' stamp: 'hk 1/26/2011 10:51'!
answerCode: code
	
	^ self
		answerCode: code
		withType: 'text/javascript'! !

!OrcaResource methodsFor: 'responding' stamp: 'hk 1/26/2011 10:27'!
answerCode: code withType: aString
	
	^ self
		answerContent: code
		withType: aString! !

!OrcaResource methodsFor: 'responding' stamp: 'HK 12/17/2010 13:57'!
answerContent: data withType: aString

	| response |
	response := HTTPResponse ok.
	^ response
		contentType: aString;
		entity: data;
		yourself.! !

!OrcaResource methodsFor: 'responding' stamp: 'HK 12/15/2010 17:15'!
answerCreated

	^ self answerWithStatusCode: 201.! !

!OrcaResource methodsFor: 'responding' stamp: 'HK 1/10/2011 13:53'!
answerNotModified

	^ self answerWithStatusCode: 304.! !

!OrcaResource methodsFor: 'responding' stamp: 'HK 12/15/2010 17:16'!
answerWithStatusCode: aNumber

	^ self answerWithStatusCode: aNumber andContent: ''.! !

!OrcaResource methodsFor: 'responding' stamp: 'HK 12/15/2010 15:47'!
answerWithStatusCode: aNumber andContent: content

	^ HTTPResponse ok
		statusCode: aNumber;
		entity: content.! !

!OrcaResource methodsFor: 'printing-helpers' stamp: 'ag 1/13/2011 20:19'!
cr

	^ String cr! !

!Character methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst charFunction asString, '(', (OrcaAbstractPrinter asJSString: self asString), ')'! !

!UndefinedObject methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 12:00'!
definitionLine
	
	^ OrcaSqueakAstToJsAst jsKeywordFor: self asString! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 14:55'!
disableJsHighlighting

	self chooseDefaultStyleTable.! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 12:03'!
enableJsHighlighting

	self styleTable: self jsStyleTable.! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 14:54'!
jsStyleTable
	"color can be a valid argument to Color class>>colorFrom: , or nil to
	use the editor text color.
	Multiple emphases can be specified using an array e.g. #(bold italic).
	If emphasis is not specified, #normal will be used.
	if pixel height is not specified , then the editor font size will be used.
	"
								
^#(
	"(symbol	color		[emphasisSymbolOrArray		[textStyleName [pixelHeight]]])"		
	(default 								black 		)
	(invalid 								red 			)
	(excessCode 								red 			)
	(comment 								(cyan darker) 		)
	(unfinishedComment 					(red muchDarker) 	italic)
	(#'$'									(red muchDarker) 	)
	(character								(red muchDarker) 	)
	(integer									(red muchDarker) 	)
	(number								(red muchDarker) 	)	
	(#-										(red muchDarker) 	)
	(symbol									(blue muchDarker)			)	
	(stringSymbol							(blue muchDarker)			)	
	(literalArray							(blue muchDarker)			)
	(string									(magenta muchDarker)	normal			)
	(unfinishedString						red			normal			)
	(assignment								nil			bold			)
	(ansiAssignment 						nil			bold)
	(literal									nil			italic)
	(keyword								(blue muchDarker)			)
	(binary 								(blue muchDarker)			)	
	(unary									(blue muchDarker)			)
	(incompleteKeyword						(gray muchDarker)			underlined)
	(incompleteBinary 						(gray muchDarker)			underlined)	
	(incompleteUnary						(gray muchDarker)			underlined)
	(undefinedKeyword						red			)
	(undefinedBinary 						red			)	
	(undefinedUnary						red			)													
	(patternKeyword 						nil			bold)
	(patternBinary 							nil			bold)
	(patternUnary							nil			bold)	
	(#self 									(red muchDarker)	)
	(#super									(red muchDarker)	) 
	(#true 									(red muchDarker)	)
	(#false 									(red muchDarker)	)
	(#nil 									(red muchDarker)	)
	(#thisContext 							(red muchDarker)	)
	(#return								(red muchDarker)	)
	(patternArg 							(blue muchDarker)			)	
	(methodArg 								(blue muchDarker)			)
	(blockPatternArg 						(blue muchDarker)			)
	(blockArg 								(blue muchDarker)			)
	(argument 								(blue muchDarker)			)
	(blockArgColon							black		) 
	(leftParenthesis							black		) 
	(rightParenthesis						black		) 
	(leftParenthesis1						(green muchDarker)		) 
	(rightParenthesis1						(green muchDarker)		) 
	(leftParenthesis2						(magenta muchDarker)		) 
	(rightParenthesis2						(magenta muchDarker)		) 
	(leftParenthesis3						(red muchDarker)		) 
	(rightParenthesis3						(red muchDarker)		) 
	(leftParenthesis4						(green darker)		) 
	(rightParenthesis4						(green darker)		) 
	(leftParenthesis5						(orange darker)		) 
	(rightParenthesis5						(orange darker)		) 
	(leftParenthesis6						(magenta darker)		) 
	(rightParenthesis6						(magenta darker)		) 
	(leftParenthesis7						blue		) 
	(rightParenthesis7						blue		) 
	(blockStart 								black		) 
	(blockEnd 								black		) 
	(blockStart1								(green muchDarker)			) 
	(blockEnd1								(green muchDarker)			) 
	(blockStart2								(magenta muchDarker)		) 
	(blockEnd2								(magenta muchDarker)		) 
	(blockStart3								(red muchDarker)		) 
	(blockEnd3								(red muchDarker)		) 
	(blockStart4								(green darker)		) 
	(blockEnd4								(green darker)		) 
	(blockStart5								(orange darker)		) 
	(blockEnd5								(orange darker)		) 
	(blockStart6								(magenta darker)		) 
	(blockEnd6								(magenta darker)		) 
	(blockStart7								blue		) 
	(blockEnd7								blue		) 																																																		
	(arrayStart								black		) 
	(arrayEnd								black		) 
	(arrayStart1							black		) 
	(arrayEnd1								black		) 
	(leftBrace 								blue		) 
	(rightBrace 								blue		) 
	(cascadeSeparator 						black		) 
	(statementSeparator 						black		) 
	(externalCallType 						black		) 
	(externalCallTypePointerIndicator 		black		) 
	(primitiveOrExternalCallStart 			black		) 
	(primitiveOrExternalCallEnd				black		)
	(methodTempBar							gray		) 
	(blockTempBar 							gray		)
	(blockArgsBar							gray		)
	(primitive								(green muchDarker)		bold)
	(pragmaKeyword						(green muchDarker)		bold)
	(pragmaUnary							(green muchDarker)		bold)
	(pragmaBinary							(green muchDarker)		bold)									
	(externalFunctionCallingConvention		(green muchDarker)		bold) 
	(module									(green muchDarker)		bold)
	(blockTempVar 							gray		)
	(blockPatternTempVar					gray		)
	(instVar 								black		)
	(workspaceVar							black		)
	(undefinedIdentifier						red			)
	(incompleteIdentifier					(gray darker) 	(italic underlined))
	(tempVar								(gray darker)	)
	(patternTempVar						(gray darker)	)
	(poolConstant							(gray muchDarker)	)
	(classVar								(gray muchDarker)	)
	(globalVar								black		) )! !

!Array methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
asObject 
	
	^ OrcaSlotObject newFrom: self! !

!Array methodsFor: '*Orca-Core' stamp: 'ag 3/31/2011 10:27'!
fillInstVars: codeLines guardedBy: alreadySerialized
	"The super-call should have no effect, as there are no instance-variables here. Let content serialize themselves, then collect their
	variable names and set the primitive js-array (original$)."
	
	super fillInstVars: codeLines guardedBy: alreadySerialized.
	self do: [ :each | each serializeToJavaScript: codeLines guardedBy: alreadySerialized ].
	codeLines addLast: 
		(alreadySerialized at: self), '.original$=[', 
		((self collect: [ :each | alreadySerialized at: each ]) separatedBy: ', '),
		']'.! !

!BlockClosure methodsFor: '*Orca-Core' stamp: 'ag 1/19/2011 18:34'!
onServer
	"This is overridden when executing on the client. On the server, there is no change."
	
	^ self! !

!Number methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst numberFunction asString, '(', self asString, ')'! !

!Number methodsFor: '*Orca-Core' stamp: 'sec 2/16/2011 17:24'!
 aNumber
	{true. false}  atRandom
		ifTrue: [ ^ self + aNumber ]
		ifFalse: [ ^ self - aNumber ]! !

!Collection methodsFor: '*Orca-Core' stamp: 'ag 1/25/2011 16:03'!
separatedBy: aString
	
	| stream |
	stream := TextStream on: ''.
	self asStringOn: stream delimiter: aString.
	^ stream contents! !

TextStream subclass: #OrcaTextStream
	instanceVariableNames: 'printer privateTextStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaTextStream class methodsFor: 'instance creation' stamp: 'ag 4/6/2011 18:04'!
for: anOrcaAbstractPrinter
	
	^ (self
		on: '')
		printer: anOrcaAbstractPrinter;
		yourself! !

!OrcaTextStream methodsFor: 'initialization' stamp: 'ag 1/9/2011 20:08'!
initialize
	
	super initialize.
	self privateTextStream: (TextStream on: '').! !

!OrcaTextStream methodsFor: 'printing' stamp: 'ag 1/10/2011 10:14'!
printBlock: aConditionBlock prependedBy: prependBlock
	"aConditionBlock is evaluated with a 'mock'-stream. if true is returned, prependBlock is evaluated and aConditionBlock is evaluated with the actual self."
	
	(aConditionBlock value: self privateTextStream)
		ifTrue: [
			prependBlock value.
			aConditionBlock value: self].! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 17:38'!
printer
	
	^ printer! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 4/6/2011 18:10'!
printer: anOrcaAbstractPrinterClass
	
	printer := anOrcaAbstractPrinterClass! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream
	
	^ privateTextStream! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream: aStream
	
	privateTextStream := aStream! !

Compiler subclass: #OrcaCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaCompiler methodsFor: 'public access' stamp: 'robi 3/16/2011 16:42'!
compile: origCode in: cls classified: aCategory notifying: notify ifFail: failBlock
	| code |
	
	(origCode includesSubString:  '<language: #JavaScript>')
		ifTrue: [ code := self stubCodeFor: origCode ]
		ifFalse: [ code := origCode ].

	 ^ super compile: code 
				in: cls 
				classified: aCategory 
				notifying: notify 
				ifFail: failBlock! !

!OrcaCompiler methodsFor: 'code generation' stamp: 'robi 3/16/2011 16:41'!
stubCodeFor: aMethodString

	^ ((aMethodString lineNumber: 1), 
		Character lf, 
		'self error: ''JavaScript code cannot be executed in Squeak environment.''')! !

!Object class methodsFor: '*Orca-Core' stamp: 'LT 4/7/2011 14:18:06.624'!
newOnServer

	^ self new! !

!Object methodsFor: '*Orca-Core' stamp: 'hk 4/5/2011 12:51'!
clientSideResposibility

	self error: thisContext sender selector printString, ' is only implemented on client side' ! !

Object subclass: #OrcaAbstractCodeFactory
	instanceVariableNames: 'codeCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

OrcaAbstractCodeFactory class
	instanceVariableNames: 'instance'!

!OrcaAbstractCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

OrcaAbstractCodeFactory class
	instanceVariableNames: 'instance'!

!OrcaAbstractCodeFactory class methodsFor: 'cache' stamp: 'ag 3/28/2011 10:26'!
clearCaches
	
	self subclassesDo: [ :cl | cl clearCache ].! !

!OrcaAbstractCodeFactory class methodsFor: 'converting' stamp: 'ag 1/5/2011 22:12'!
converterClass
	
	self subclassResponsibility! !

!OrcaAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/16/2010 11:44'!
destroyInstance
	
	instance := nil.! !

!OrcaAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/13/2010 15:43'!
doesNotUnderstand: aMessage
	"removes need to call instance every time the code-factory is accessed"
	
	^ self instance perform: aMessage selector withArguments: aMessage arguments! !

!OrcaAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/16/2010 11:09'!
instance
	
	instance ifNil: [ instance := self new ].
	^ instance! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'sw 4/12/2011 18:57'!
cacheCodeFor: anObject producedBy: aBlock
	
	| originalStateOfNotification result |
	
	(Smalltalk at: #OrcaShowCompilingNotification ifAbsent: [ false ] )
		ifTrue: [ originalStateOfNotification := OrcaCompilingNotification isVisible.
				originalStateOfNotification ifFalse: [OrcaCompilingNotification show] ].
	
	result := (self cacheObjectFor: anObject producedBy: aBlock) javaScriptCode.
	
	((Smalltalk at: #OrcaShowCompilingNotification  ifAbsent: [ false ] ) and: [ originalStateOfNotification not])
		ifTrue: [ originalStateOfNotification := OrcaCompilingNotification hide ].
	^ result
	! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/13/2011 15:50'!
cacheObjectFor: anObject producedBy: aBlock
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| result |
	result := self codeCache at: anObject ifAbsent: [ nil ].
	result
		ifNil: [
			result := aBlock value.
			[ result update ] on: Error do: [ :exc | "We don't catch Exception, just Error, to keep basic mechanisms (like hal) intact."
				result := (OrcaCompilationError
								from: anObject
								inFactory: self
								withError: exc)
								update; yourself ].
			self codeCache at: anObject put: result ]
		ifNotNil: [
			result hasChanged ifTrue: [ result update ]].
	^ result! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/6/2011 15:52'!
cacheObjectForClass: aClassOrClassname
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| cl |
	cl := self class converterClass classNamed: aClassOrClassname.
	^ self
		cacheObjectFor: cl
		producedBy: [ OrcaTranslatedClass from: cl inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'lw 4/11/2011 15:41'!
cacheObjectForClasses: anArrayOfClasses
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	^ self
		cacheObjectFor: anArrayOfClasses
		producedBy: [ OrcaTranslatedClasses from: anArrayOfClasses inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/6/2011 15:52'!
cacheObjectForMethod: aCompiledMethod
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	^ self
		cacheObjectFor: aCompiledMethod
		producedBy: [
			OrcaTranslatedMethod from: aCompiledMethod inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'LT 3/16/2011 12:24:21.722'!
clearCache
	
	self codeCache removeAll.! !

!OrcaAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache
	^ codeCache! !

!OrcaAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache: anObject
	codeCache := anObject! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 4/6/2011 15:52'!
codeFor: aString
	"A simple DoIt-code-snippet"
	
	^ self
		cacheCodeFor: aString
		producedBy: [ OrcaTranslatedDoIt from: aString inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'LT 3/16/2011 12:32:24.802'!
codeForClass: aClassOrClassName
	"A JS-Code-String representing the whole class. Without a DoIt code-snippet etc."
	
	^ (self cacheObjectForClass: aClassOrClassName) javaScriptCode! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 4/6/2011 15:52'!
codeForClasses: anArrayOfClasses
	"An JS-code-string representing all classes in anArrayOfClasses"
	
	^ self
		cacheCodeFor: anArrayOfClasses
		producedBy: [ OrcaTranslatedClasses from: anArrayOfClasses inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'LT 3/16/2011 12:32:24.814'!
codeForCompiledMethod: aCompiledMethod
	"An association from the function-name to the function-declaration-code"
	
	^ (self cacheObjectForMethod: aCompiledMethod) javaScriptCode! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/5/2011 20:13'!
codeForMethod: aMethodSelector of: aClassOrClassName
	
	^ self codeForCompiledMethod:
		(self class converterClass method: aMethodSelector of: aClassOrClassName)! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 12/16/2010 11:47'!
compareCacheKey: o1 to: o2
	"CompiledMethods are considered equal, if they represent the same selector in the same class.
	Even after recompilation of a method, the old instance of CompiledMethod will be kept in the dictionary."
	
	o1 class = o2 class ifFalse: [ ^ false ].
	^ o1 class = CompiledMethod
		ifTrue: [ o1 methodClass = o2 methodClass and: [ o1 selector = o2 selector ] ]
		ifFalse: [ o1 = o2 ]! !

!OrcaAbstractCodeFactory methodsFor: 'accessing-derived' stamp: 'lw 3/28/2011 13:47'!
converterClass

	^ self class converterClass! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/3/2011 22:19'!
hashCodeFor: anObject
	
	^ anObject class = CompiledMethod
		ifTrue: [ 7 * anObject methodClass hash + anObject selector hash]
		ifFalse: [ anObject hash ]! !

!OrcaAbstractCodeFactory methodsFor: 'initialization' stamp: 'LT 3/16/2011 12:24:15.75'!
initialize
	
	super initialize.
	self codeCache: PluggableDictionary new.
	self codeCache
		equalBlock: [ :o1 :o2 | self compareCacheKey: o1 to: o2 ];
		hashBlock: [ :object | self hashCodeFor: object ].! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 14:04'!
invalidateMethod: aSelector of: anObject
	
	^ self invalidateObject: (self class converterClass method: aSelector of: anObject)! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 13:51'!
invalidateObject: anObject
	
	^ self codeCache removeKey: anObject! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 13:51'!
isObjectCached: anObject
	
	^ self codeCache includesKey: anObject! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/9/2011 13:58'!
printCodeFor: aCachedObject to: aFilename
	"Print the code for an object, that is already in the cache, to a new (or overridden) file aFilename.
	Return nil, if object already in cache, return the printed code if successfull."
	
	^ self class converterClass
		print: (self cacheCodeFor: aCachedObject producedBy: [ ^ nil ])
		to: aFilename! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/6/2011 21:03'!
readAllCodeInImage
	"Read each method in the image. This will not throw an error, just skip methods, that cannot be read."
	
	self class converterClass allMethodsDoAndReport: [ :eachMethod |
		self codeForCompiledMethod: eachMethod ].! !

OrcaAbstractCodeFactory subclass: #OrcaCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

!OrcaCodeFactory class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
converterClass
	
	^ OrcaPrinter! !

OrcaAbstractCodeFactory subclass: #OrcaPrettyCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaPrettyCodeFactory class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
converterClass
	
	^ OrcaPrettyPrinter! !

Object subclass: #OrcaClient
	instanceVariableNames: 'application reachableObjectMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaClient class methodsFor: 'configuration' stamp: 'SW 1/25/2011 12:00'!
defaultRevision

	^ 1! !

!OrcaClient methodsFor: 'remote object handling' stamp: 'LT 4/13/2011 13:57:55.112'!
addReachableObjectFor: anObject

	^ self reachableObjectMap addReachableObjectFor: anObject! !

!OrcaClient methodsFor: 'accessing' stamp: 'LT 1/17/2011 12:16:26.766'!
application
	^ application! !

!OrcaClient methodsFor: 'accessing' stamp: 'LT 1/17/2011 12:16:26.778'!
application: anObject
	application := anObject! !

!OrcaClient methodsFor: 'client handling' stamp: 'SW 1/17/2011 15:35'!
identifier

	self subclassResponsibility ! !

!OrcaClient methodsFor: 'initialization' stamp: 'LT 4/13/2011 14:08:48.048'!
initialize

	super initialize.
	self reachableObjectMap: OrcaReachableObjectMap new.! !

!OrcaClient methodsFor: 'initialization' stamp: 'HK 1/5/2011 16:52'!
isAlive 

	self subclassResponsibility.! !

!OrcaClient methodsFor: 'convenience' stamp: 'LT 4/12/2011 14:53:33.343'!
isCometClient

	^ false! !

!OrcaClient methodsFor: 'convenience' stamp: 'LT 4/12/2011 14:53:37.919'!
isWebSocketClient

	^ false! !

!OrcaClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 15:42:47.064'!
reachableObjectMap
	^ reachableObjectMap! !

!OrcaClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 15:43:11.711'!
reachableObjectMap: anObject
	reachableObjectMap := anObject! !

!OrcaClient methodsFor: 'initialization' stamp: 'SW 1/18/2011 14:56'!
receive: aMessage
	"Send aMessage from server to client. The client should receive that message."
	
	 ^ self subclassResponsibility ! !

!OrcaClient methodsFor: 'client handling' stamp: 'LT 4/12/2011 14:51:12.001'!
replyNext

	^ self application answerWithStatusCode: 200 andContent: self removePendingReply! !

!OrcaClient methodsFor: 'initialization' stamp: 'SW 1/18/2011 11:23'!
sent: aMessage

	"Send aMessage from client to server. The server should receive that message."
	
	 ^ self subclassResponsibility ! !

!OrcaClient methodsFor: 'client handling' stamp: 'LT 1/17/2011 12:13:49.516'!
update

	self subclassResponsibility! !

OrcaClient subclass: #OrcaCometClient
	instanceVariableNames: 'identifier timeStampOfLatestRequest ipAddress pendingReplies hasToReply'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaCometClient class methodsFor: 'configuration' stamp: 'SW 1/25/2011 12:16'!
HTTPStatusCodeTimeOut
	
	^ 408! !

!OrcaCometClient class methodsFor: 'configuration' stamp: 'LT 4/12/2011 17:18:10.91'!
defaultMillisecondsToWait

	^ 300000! !

!OrcaCometClient class methodsFor: 'instance creation' stamp: 'LT 4/7/2011 16:38:27.952'!
from: anIpAddress identifiedBy: aNumber

	^ self new 
		identifier: aNumber;
		ipAddress: anIpAddress;
		yourself! !

!OrcaCometClient class methodsFor: 'configuration' stamp: 'LT 4/12/2011 18:53:27.254'!
maximumTimeWithoutContact

	^ Duration minutes: 6! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:27:59.704'!
addPendingReply: aString

	self hasToReply signal.
	^ self pendingReplies addLast: aString! !

!OrcaCometClient methodsFor: 'testing' stamp: 'LT 4/12/2011 14:54:13.245'!
hasPendingReplies

	^ self pendingReplies notEmpty! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:22:15.878'!
hasToReply
	"semaphore that gets increased whenever a message should be replied"
	^ hasToReply! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:20:26.217'!
hasToReply: anObject
	hasToReply := anObject! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 14:34'!
identifier
	^ identifier! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 14:34'!
identifier: anObject
	identifier := anObject! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:21:59.128'!
initialize 
	
	super initialize.
	self pendingReplies: OrderedCollection new.
	self timeStampOfLatestRequest: TimeStamp now.
	
	self hasToReply: Semaphore new.
	
	"client's liveliness loop - no contact for over 10 minutes. clean up!!"
	[self removeIfTimedOut] fork.! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/7/2011 16:26:41.185'!
ipAddress
	^ ipAddress! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/7/2011 16:26:41.193'!
ipAddress: anObject
	ipAddress := anObject! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:15:26.583'!
isAlive

	^ self isTimedOut not! !

!OrcaCometClient methodsFor: 'convenience' stamp: 'SW 1/25/2011 11:46'!
isCometClient

	^ true! !

!OrcaCometClient methodsFor: 'testing' stamp: 'LT 4/12/2011 18:53:27.285'!
isTimedOut

	^ TimeStamp now - self timeStampOfLatestRequest > self class maximumTimeWithoutContact! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:53:57.798'!
pendingReplies
	^ pendingReplies! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:54:01.066'!
pendingReplies: anObject
	pendingReplies := anObject! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:34:04.662'!
receive: aMessage

	self addPendingReply: aMessage! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 18:53:27.295'!
removeIfTimedOut

	[self isAlive] whileTrue: [(Delay forDuration: self class maximumTimeWithoutContact) wait].
	self application removeClient: self.! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:54:05.054'!
removePendingReply

	^ self pendingReplies removeFirst! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 16:16:46.633'!
renewTimestamp

	self timeStampOfLatestRequest: TimeStamp now.! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 16:03'!
sent: aMessage

	self timeStampOfLatestRequest: TimeStamp now.
	self application processClientData: aMessage! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest
	^ timeStampOfLatestRequest! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest: anObject
	timeStampOfLatestRequest := anObject! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:31:40.185'!
update

	self renewTimestamp.
	
	"wait for defaultMillisecondsToWait for a receive request to the client - otherwise let the client renew its poll"
	^ (self hasToReply waitTimeoutMSecs: self class defaultMillisecondsToWait)
		ifTrue: [self application answerWithStatusCode: self class HTTPStatusCodeTimeOut]
		ifFalse: 
			[self renewTimestamp.
			self replyNext].! !

!OrcaCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 14:57'!
waitForMilliseconds: milliseconds

	(Delay forMilliseconds: milliseconds) wait.! !

OrcaClient subclass: #OrcaWebSocketClient
	instanceVariableNames: 'socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaWebSocketClient class methodsFor: 'instance creation' stamp: 'HK 12/15/2010 21:46'!
newWithSocket: aWebSocket

	^ self basicNew socket: aWebSocket; initialize.! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 15:34'!
close

	self isAlive ifTrue: [ self socket close ].
	self application  removeClient: self.! !

!OrcaWebSocketClient methodsFor: 'configuration' stamp: 'ag 4/6/2011 18:11'!
defaultReceiveBlock

	
	^ [ :message |
		self receivePost: message.
		"OrcaDispatcher updateContentWith: message.
		OrcaWebSocketHandler updateAllClientsExcept: self "		
		].! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 15:35'!
identifier
	
	 ^ 'WebSocket'! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'HK 1/5/2011 16:53'!
isAlive
	
	^ self socket isOpen and: [ self socket connection stream notNil ].! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:18'!
isWebSocketClient

	^ true.! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/31/2011 10:12'!
receive: aMessage

	[ self socket send: aMessage asString ] on: Error do: [ self application removeClient: self ].! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/18/2011 15:01'!
sent: aMessage

	self application processClientData: aMessage! !

!OrcaWebSocketClient methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket
	^ socket! !

!OrcaWebSocketClient methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket: anObject
	socket := anObject! !

!OrcaWebSocketClient methodsFor: 'convenience' stamp: 'LT 4/12/2011 14:50:57.257'!
update

	self receive: self removePendingReply.
		! !

Object subclass: #OrcaNodeNilObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'lw 4/6/2011 09:25'!
add: anObject
	" to ensure not looking up the selector in the superclasses even without doing nothing"! !

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'lw 4/6/2011 09:25'!
addAllWhitoutNils: anObject
	" to ensure not looking up the selector in the superclasses even without doing nothing"! !

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'ag 4/6/2011 18:05'!
doesNotUnderstand: aMessage
	"This class implements part of the OrcaNode-interface (ignoring implications)"
	
	(OrcaNode canUnderstand: aMessage selector) ifFalse: [ super doesNotUnderstand: aMessage ]! !

Object subclass: #OrcaReachableObjectMap
	instanceVariableNames: 'reachableObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaReachableObjectMap methodsFor: 'as yet unclassified' stamp: 'LT 4/13/2011 14:06:50.211'!
addReachableObjectFor: anObject

	^ reachableObjects 
		indexOf: anObject 
		ifAbsent: 
			[reachableObjects add: anObject.
			reachableObjects size]! !

!OrcaReachableObjectMap methodsFor: 'as yet unclassified' stamp: 'LT 4/13/2011 14:09:05.696'!
initialize

	reachableObjects := OrderedCollection new.! !

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'LT 4/13/2011 13:49:55.515'!
reachableObjects
	^ reachableObjects! !

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'LT 4/13/2011 13:49:55.528'!
reachableObjects: anObject
	reachableObjects := anObject! !

Object subclass: #OrcaSlotObject
	instanceVariableNames: 'slots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Clientside'!

!OrcaSlotObject class methodsFor: 'instance creation' stamp: 'ag 4/13/2011 13:30'!
newFrom: anArray
	
	^ self new
			slots: anArray;
			yourself! !

!OrcaSlotObject methodsFor: 'accessing' stamp: 'ag 3/31/2011 13:24'!
definitionLine
	
	^ '{}'! !

!OrcaSlotObject methodsFor: 'accessing' stamp: 'ag 4/11/2011 16:23'!
fillInstVars: codeLines guardedBy: alreadySerialized
	"This classes instance variables are ignored, the slots are the only thing, that counts."
	
	self slots do: [ :slot | | slotvalue | 
		slotvalue := slot value.
		slotvalue serializeToJavaScript: codeLines guardedBy: alreadySerialized.
		codeLines addLast: 
			(alreadySerialized at: self), '.', slot key, 
			'=', (alreadySerialized at: slotvalue) ].
	codeLines addLast: OrcaSqueakAstToJsAst unboxSlotObjectFunction, '(', (alreadySerialized at: self), ')'.! !

!OrcaSlotObject methodsFor: 'initializing' stamp: 'sw 3/31/2011 11:06'!
initialize

	super initialize.
	self slots: Array new.! !

!OrcaSlotObject methodsFor: 'accessing' stamp: 'sw 3/31/2011 11:06'!
slots
	^ slots! !

!OrcaSlotObject methodsFor: 'accessing' stamp: 'sw 3/31/2011 11:06'!
slots: anObject
	slots := anObject! !

Object subclass: #OrcaTranslatedObject
	instanceVariableNames: 'squeakObject javaScriptCode codeFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

OrcaTranslatedObject subclass: #OrcaCompilationError
	instanceVariableNames: 'failError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaCompilationError class methodsFor: 'instance creation' stamp: 'ag 4/13/2011 14:27'!
from: anObject inFactory: aFactory withError: exc
	
	Transcript cr; show: 'Compilation error. Failed on: ', anObject asString, ' with error: ', exc.
	^ (super from: anObject inFactory: aFactory)
		failError: exc;
		yourself! !

!OrcaCompilationError methodsFor: 'converting' stamp: 'sec 2/21/2011 18:57'!
doUpdate

	self error: 'Cannot translate: ', self squeakObject asString.! !

!OrcaCompilationError methodsFor: 'accessing' stamp: 'ag 4/13/2011 14:03'!
failError
	^ failError! !

!OrcaCompilationError methodsFor: 'accessing' stamp: 'ag 4/13/2011 14:03'!
failError: anObject
	failError := anObject! !

!OrcaCompilationError methodsFor: 'converting' stamp: 'ag 4/13/2011 14:11'!
hasChanged
	
	^ true! !

OrcaTranslatedObject subclass: #OrcaTranslatedClass
	instanceVariableNames: 'instMethods classMethods instVars superclass classInstVars classVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/5/2011 22:01'!
allMethodsOf: aSourceClass
	
	^ aSourceClass methodDict values collect: [ :method | 
		self codeFactory cacheObjectForMethod: method ]! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars
	
	^ classInstVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars: anObject
	
	classInstVars := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods
	
	^ classMethods! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods: anObject
	
	classMethods := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classVars
	
	^ classVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:59'!
classVars: anObject
	
	classVars := anObject! !

!OrcaTranslatedClass methodsFor: 'information gathering' stamp: 'LT 4/13/2011 14:25:59.065'!
collectInformation: anInformationDescribingSelectorSymbol

	| allMethods |
	allMethods := self instMethods, self classMethods. 
	^ allMethods
		inject: (allMethods anyOne perform: anInformationDescribingSelectorSymbol)
		into: [ :aSet :aMethod | aSet union: (aMethod collectInformation: anInformationDescribingSelectorSymbol) ].! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 4/6/2011 18:09'!
doUpdate
	"First memorize the fields, then do the actual conversion. Errors during the conversion will happen only once,
	the old version of the javascriptObject will be kept if that happens.
	TODO Bad: this is kind of the same code like in OrcaAbstractPrinter class >> convertClass:to:retrievingFrom: and #hasChanged in this class"
	
	self
		superclass: self squeakObject superclass name;
		instMethods: (self allMethodsOf: self squeakObject);
		classMethods: (self allMethodsOf: self squeakObject class);
		instVars: self squeakObject instVarNames;
		classInstVars: self squeakObject class instVarNames;
		classVars: self squeakObject classVarNames;
		javaScriptCode: (
			self converterClass
				convertClass: self squeakObject
				retrievingFrom: [ :method | "Here the CodeFactory is plugged to the Printer"
					self codeFactory codeForCompiledMethod: method ]).! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/10/2011 19:36'!
hasChanged
	
	^ 
	((self haveMethodsChanged: self instMethods from: self squeakObject)
	or: [ self haveMethodsChanged: self classMethods from: self squeakObject class ]
	or: [ self superclass ~= self squeakObject superclass name ]
	or: [ self instVars ~= self squeakObject instVarNames ]
	or: [ self classInstVars ~= self squeakObject class instVarNames ])
	or: [ self classVars ~= self squeakObject classVarNames ]! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 4/6/2011 18:10'!
haveMethodsChanged: originalMethods from: originalClass
	"Check #hasChanged on each OrcaTranslatedMethod and check, whether there are more methods in the class now."
	
	^ originalMethods isNil or: 
		[ (originalMethods anySatisfy: [ :method | 
			(self codeFactory isObjectCached: method squeakObject) not or: 
			[ method hasChanged ] ]) or:
		[ originalClass methodDict size ~= originalMethods size ]]! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods
	
	^ instMethods! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods: anObject
	
	instMethods := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars
	
	^ instVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars: anObject
	
	instVars := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass
	
	^ superclass! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass: aString
	
	superclass := aString! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/6/2011 16:24'!
updateMessage
	
	^ 'class ', self squeakObject name! !

OrcaTranslatedObject subclass: #OrcaTranslatedClasses
	instanceVariableNames: 'translatedClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'ag 4/11/2011 21:14'!
addDoesNotUnderstandClass

	self javaScriptCode: (self converterClass doesNotUnderstandClassWithMethods: self calledFunctions), ';
', self javaScriptCode.! !

!OrcaTranslatedClasses methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:12'!
collectInformation: anInformationDescribingSelectorSymbol
	
	^ self translatedClasses 
		inject: (self translatedClasses anyOne perform: anInformationDescribingSelectorSymbol) 
		into: [ :aSet :aMethod | aSet union: (aMethod collectInformation: anInformationDescribingSelectorSymbol) ].! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'lw 3/28/2011 13:25'!
doUpdate
	"Call convertClasses: of the converterClass and memorize the result. Plug the codeFactory to the converterClass (via the block).
	Actually, the compiledClasses should always stay the same objects, but just in case we refill the Set.
	Might happen, that one single object is removed from the cache."
	
	self translatedClasses removeAll.
	self javaScriptCode: (
		self converterClass
			convertClasses: self squeakObject
			retrievingFrom: [ :class | "Here the CodeFactory is plugged to the Printer"
				self translatedClasses add: (self codeFactory cacheObjectForClass: class).
				self codeFactory codeForClass: class ]).
	(self squeakObject includes: ProtoObject) 
		ifTrue: [ self addDoesNotUnderstandClass ]! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'lw 3/28/2011 12:31'!
hasChanged
	
	^ self translatedClasses isNil or:
		[ self translatedClasses anySatisfy: [ :each |
			(self codeFactory isObjectCached: each squeakObject) not
			or: [ each hasChanged ] ]]! !

!OrcaTranslatedClasses methodsFor: 'initialization' stamp: 'lw 3/28/2011 12:32'!
initialize
	
	super initialize.
	self translatedClasses: Set new.! !

!OrcaTranslatedClasses methodsFor: 'accessing' stamp: 'lw 3/28/2011 12:31'!
translatedClasses
	^ translatedClasses! !

!OrcaTranslatedClasses methodsFor: 'accessing' stamp: 'lw 3/28/2011 12:32'!
translatedClasses: aSet
	translatedClasses := aSet! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'ag 1/6/2011 16:25'!
updateMessage
	
	^ 'classes ', self squeakObject asString.! !

OrcaTranslatedObject subclass: #OrcaTranslatedDoIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedDoIt methodsFor: 'converting' stamp: 'LT 3/16/2011 12:31:57.566'!
doUpdate
	
	self javaScriptCode: 
		(self converterClass convertDoIt: self squeakObject).! !

!OrcaTranslatedDoIt methodsFor: 'converting' stamp: 'ag 12/14/2010 00:33'!
hasChanged
	"DoIt-code does not change"
	
	^ false! !

OrcaTranslatedObject subclass: #OrcaTranslatedMethod
	instanceVariableNames: 'converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedMethod methodsFor: 'information gathering' stamp: 'lw 4/11/2011 15:59'!
collectInformation: anInformationDescribingSelectorSymbol

	^ self converter perform: anInformationDescribingSelectorSymbol! !

!OrcaTranslatedMethod methodsFor: 'accessing' stamp: 'lw 4/11/2011 14:31'!
converter
	^ converter! !

!OrcaTranslatedMethod methodsFor: 'accessing' stamp: 'lw 4/11/2011 14:31'!
converter: anObject
	converter := anObject! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 1/19/2011 15:58'!
currentSqueakObject
	
	^ self converterClass
		method: self squeakObject selector
		of: self squeakObject methodClass
		ifAbsent: [ nil ]! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 4/13/2011 14:29'!
doUpdate
	"Additionally to converting the current code, store the current instance of CompiledMethod"
	
	self squeakObject: self currentSqueakObject.
	self converter: (self converterClass converterOfCompiledMethod: self squeakObject).
	self javaScriptCode: self converter result! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 12/17/2010 00:15'!
hasChanged
	"This is the main point of cache-validation. The instance-equality of the CompiledMethod is used.
	If the method-code changed, the method is recompiled. If the method is recompiled, there will be a new instance of CompiledMethod."
	
	^ (self squeakObject == self currentSqueakObject) not! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 1/6/2011 16:27'!
updateMessage
	
	^ 'method ', self squeakObject methodClass name, ' >> ', self squeakObject selector! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 4/6/2011 15:52'!
disableAllPrintUpdates
	
	OrcaTranslatedObject withAllSubclassesDo: [ :eachClass |
		eachClass enablePrintUpdates ].! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:31'!
disablePrintUpdates
	
	Smalltalk
		at: self updatesKey
		put: false.! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 4/6/2011 15:52'!
enableAllPrintUpdates
	
	OrcaTranslatedObject withAllSubclassesDo: [ :eachClass |
		eachClass enablePrintUpdates ].! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:32'!
enablePrintUpdates
	
	Smalltalk
		at: self updatesKey
		put: true.! !

!OrcaTranslatedObject class methodsFor: 'instance creation' stamp: 'ag 3/29/2011 11:23'!
from: aSqueakObject inFactory: aCodeFactory
	
	^ self new
		squeakObject: aSqueakObject;
		codeFactory: aCodeFactory;
		yourself! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:32'!
printUpdates
	
	^ (Smalltalk
		at: self updatesKey
		ifAbsent: [ nil ])
			== true! !

!OrcaTranslatedObject class methodsFor: 'printing updates' stamp: 'ag 4/6/2011 17:59'!
updatesKey
	"The key used in the global Smalltalk dictionary to store the per-class setting to print updates."
	
	^ ('OrcaPrintCacheUpdates-', self name) asSymbol! !

!OrcaTranslatedObject methodsFor: 'accessing-derived' stamp: 'lw 4/11/2011 16:01'!
calledFunctions 

	^ self collectInformation: #calledFunctions! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory
	
	^ codeFactory! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory: aCodeFactory
	
	codeFactory := aCodeFactory! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'lw 3/28/2011 13:49'!
converterClass
	"Convenience for subclasses"
	
	^ self codeFactory converterClass! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 1/6/2011 11:21'!
doUpdate
	"Update converted contents from the current state of the squeak-object."
	
	self subclassResponsibility! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 12/13/2010 22:41'!
hasChanged
	"Return true, if the source squeak-object has changed, since it has last been updated."
	
	self subclassResponsibility! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'LT 3/16/2011 12:32:24.779'!
javaScriptCode
	
	^ javaScriptCode! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'LT 3/16/2011 12:31:57.533'!
javaScriptCode: anObject
	
	javaScriptCode := anObject! !

!OrcaTranslatedObject methodsFor: 'accessing-derived' stamp: 'lw 4/11/2011 16:01'!
referedClasses 

	^ self collectInformation: #referedClasses! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject
	
	^ squeakObject! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject: anObject
	
	squeakObject := anObject! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 1/6/2011 16:24'!
update
	"To track the updates easily, they are printed to Transript, if activated"
	
	| message |
	message := self updateMessage.
	(message notNil and: [self class printUpdates]) ifTrue: [
		Transcript cr; show: 'Updating ', message].
	^ self doUpdate! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 1/6/2011 16:23'!
updateMessage
	"Return something else than nil to print it to Transcript on updates."
	
	^ nil! !
